﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Drawing;
using System.ComponentModel;
using System.Xml.Serialization;

namespace NauPlaying
{
	/// <summary>
	/// アプリケーション設定
	/// </summary>
	[TypeConverter(typeof(PropertyDisplayConverter))]
	public class AppSetting
	{
		/// <summary>
		/// ボード表示位置
		/// </summary>
		[PropertyDisplayName("01 : 表示位置")]
		[Description("ボード表示位置"), Category("01 : 表示設定")]
		public Point BoardLocation { set; get; }
		/// <summary>
		/// テキスト表示フォーマット
		/// </summary>
		[PropertyDisplayName("02 : 表示フォーマット")]
		[Description("曲情報として表示されるテキストのフォーマット\n" +
			"以下の制御文字が使用できます。（半角英数）\n" +
			"%A : アーティスト名\n" +
			"%L : アルバム名\n" +
			"%T : トラック名\n" +
			"%I : トラック番号\n" +
			"%N : 改行"), Category("01 : 表示設定")]
		public string TextFormat { set; get; }
		/// <summary>
		/// 表示する際の倍率
		/// </summary>
		[PropertyDisplayName("03 : 表示倍率")]
		[Description("表示される際の倍率を指定します。"), Category("01 : 表示設定")]
		public float ZoomMaginification { set; get; }
		/// <summary>
		/// ボード不透明度 (1 不透明 ～ 0 透明)
		/// </summary>
		[PropertyDisplayName("04 : ボード不透明度")]
		[Description("表示される際の不透明度を 1:不透明～0:透明の間で設定します。"), Category("01 : 表示設定")]
		public float BoardAlpah { set; get; }
		/// <summary>
		/// アートワークが存在しない場合の表示イメージファイル名
		/// </summary>
		[PropertyDisplayName("05 : 標準アートワーク")]
		[Description("アートワークが存在しない場合の表示イメージファイル名を設定します。\n（使用しない場合は空にしてください。アプリケーションが用意する標準のイメージを使用します。）"), Category("01 : 表示設定")]
		public string NoneArtworkImageFilename { set; get; }
		/// <summary>
		/// アートワークが存在しない場合 NoneArtworkImageFilename へ設定したイメージを表示する
		/// </summary>
		[PropertyDisplayName("06 : 標準アートワークの使用/不使用")]
		[Description("true : アートワークが存在しない場合 NoneArtworkImageFilename へ設定したイメージを表示します。\nfalse : 表示しない。（透過されます）"), Category("01 : 表示設定")]
		public bool DispNoneArtworkImage { set; get; }
		/// <summary>
		/// iTunes ボード表示位置
		/// </summary>
		[PropertyDisplayName("07 : iTunes 表示位置")]
		[Description("iTunes ボード表示位置\n「09 : プレーヤでの表示位置の切り替え」=true で有効です。 false の場合は、「01 : 表示位置」が使用されます。"), Category("01 : 表示設定")]
		public Point iTunesBoardLocation { set; get; }
		/// <summary>
		/// Winamp ボード表示位置
		/// </summary>
		[PropertyDisplayName("08 : Winamp 表示位置")]
		[Description("Winamp ボード表示位置\n「09 : プレーヤでの表示位置の切り替え」=true で有効です。 false の場合は、「01 : 表示位置」が使用されます。"), Category("01 : 表示設定")]
		public Point WinampBoardLocation { set; get; }
		/// <summary>
		/// プレーヤーごとに設定ファイルを入れ替えるの設定。
		/// </summary>
		[PropertyDisplayName("09 : プレーヤでの表示位置の切り替え")]
		[Description("プレーヤー専用のボード表示位置使用するかの設定。\ntrue : 行う、false : 行わない。"), Category("01 : 表示設定")]
		public bool EnablePlayerBoardLocation { set; get; }

		/// <summary>
		/// テキストフォント
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("01 : テキストフォント")]
		[Description("テキストフォント"), Category("02 : テキスト設定")]
		public Font TextFont { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string TextFontAsString
		{
			get { return Generic.ConvertToString(TextFont); }
			set { TextFont = Generic.ConvertFromString<Font>(value); }
		}
		/// <summary>
		/// テキスト色1
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("02 : テキスト色1")]
		[Description("テキスト色1"), Category("02 : テキスト設定")]
		public Color TextColor1 { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string TextColor1AsString
		{
			get { return Generic.ConvertToString(TextColor1); }
			set { TextColor1 = Generic.ConvertFromString<Color>(value); }
		}
		/// <summary>
		/// テキスト色2
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("03 : テキスト色2")]
		[Description("テキスト色2"), Category("02 : テキスト設定")]
		public Color TextColor2 { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string TextColor2AsString
		{
			get { return Generic.ConvertToString(TextColor2); }
			set { TextColor2 = Generic.ConvertFromString<Color>(value); }
		}
		/// <summary>
		/// テキストの描画モード
		/// </summary>
		[PropertyDisplayName("04 : テキスト描画モード")]
		[Description(
			"テキストの描画モード\n" +
			"Normal : テキスト色1で普通に描画。\n" +
			"Outline : アウトライン（縁）のみをテキスト色2で描画。\n" +
			"Border : フチ文字描画。文字部分をテキスト色1、縁をテキスト色2で描画。\n" +
			"Shadow : 影つき文字描画。文字部分をテキスト色1、影をテキスト色2で描画。"), Category("02 : テキスト設定")]
		public TextDrawingMode TextDrawingMode { set; get; }
		/// <summary>
		/// テキストアウトライン表示幅
		/// </summary>
		[PropertyDisplayName("05 : アウトライン表示幅")]
		[Description(
			"テキストのアウトライン線幅 1 ～ 10で設定\n" +
			"TextDrawingMode でどのモードを選択しているかで意味が異なります。\n" +
			"Normal : 使用されません。\n" +
			"Outline : アウトライン（縁）の描画幅となります。\n" +
			"Border : アウトライン（縁）の描画幅となります。\n" +
			"Shadow : 影のずれとして使用されます。\n" +
			"BorderAndShadow : アウトライン幅と 影のずれとして使用されます。"), Category("02 : テキスト設定")]
		public int OutlineWidth { set; get; }
		/// <summary>
		/// スキン側のテキスト描画設定を使用するか
		/// </summary>
		[PropertyDisplayName("06 : スキン側のテキスト描画設定を使用する")]
		[Description("スキン側のテキスト描画設定を使用するかの設定。\ntrue : スキン設定のテキスト描画設定を使用する。\nfalse : アプリケーション設定のテキスト描画設定を使用する。"), Category("02 : テキスト設定")]
		public bool UseSkinTextDrawingSetting { set; get; }
		/// <summary>
		/// スキン側のフォントを使用するか
		/// </summary>
		[PropertyDisplayName("07 : スキン側のフォント設定を使用する")]
		[Description("スキン側のフォント設定を使用するかの設定。\ntrue : スキン設定のフォント設定を使用する。\nfalse : アプリケーション設定のフォント設定を使用する。"), Category("02 : テキスト設定")]
		public bool UseSkinTextFont { set; get; }

		/// <summary>
		/// フェードイン時間[ms]
		/// </summary>
		[PropertyDisplayName("01 : フェードイン時間[ms]")]
		[Description("フェードイン時間[ms]\n0～10000[ms]で設定。フェードインを無効にしたい場合は 0 を設定。"), Category("03 : アニメーション設定")]
		public int FadeinTime { set; get; }
		/// <summary>
		/// フェードアウト時間[ms]
		/// </summary>
		[PropertyDisplayName("02 : フェードアウト時間[ms]")]
		[Description("フェードアウト時間[ms]\n0～10000[ms]で設定。フェードアウトを無効にしたい場合は 0 を設定。"), Category("03 : アニメーション設定")]
		public int FadeoutTime { set; get; }
		/// <summary>
		/// スクロールモード
		/// </summary>
		[PropertyDisplayName("03 : スクロールモード")]
		[Description(
			"スクロールモード\n" +
			"None : スクロールしない。\n" +
			"Mode1 : 前進スクロール。\n" +
			"Mode2 : 前進→後進で往復スクロール。"), Category("03 : アニメーション設定")]
		public ScrollMode ScrollMode { set; get; }
		/// <summary>
		/// スクロールの1ステップの移動量[px]
		/// </summary>
		[PropertyDisplayName("04 : 1ステップの移動量[px]")]
		[Description("スクロール時の1ステップの移動量[px]\n1～100で設定。小さくすると遅く細かく、大きくすると早く荒く動く。"), Category("03 : アニメーション設定")]
		public int ScrollStep { set; get; }
		/// <summary>
		/// スクロール動作後の停止時間[ms]
		/// </summary>
		[PropertyDisplayName("05 : 動作後の停止時間[ms]")]
		[Description("スクロール動作後の停止時間[ms]\n0～10000[ms]で設定。スクロール動作後のため時間。"), Category("03 : アニメーション設定")]
		public int ScrollStopTime { set; get; }
		/// <summary>
		/// スクロールしない場合の停止時間[ms]
		/// </summary>
		[PropertyDisplayName("06 : スクロールしない場合の停止時間[ms]")]
		[Description("スクロールしない場合の停止時間[ms]\n0～10000[ms]で設定。スクロールしない場合や、テキストが短くてスクロールする必要がない場合に使用される。"), Category("03 : アニメーション設定")]
		public int NoneScrollStopTime { set; get; }
		/// <summary>
		/// アニメーションステップタイム[ms]
		/// </summary>
		[PropertyDisplayName("07 : ステップタイム[ms]")]
		[Description("アニメーションステップタイム[ms]\n10～1000[ms]で設定。スクロールの早さも変わるので注意。\n＊ アニメーション実行間隔を短くすると負荷が重くなるので注意。"), Category("03 : アニメーション設定")]
		public int StepTime { set; get; }
		/// <summary>
		/// スクロール加速度
		/// </summary>
		[PropertyDisplayName("08 : スクロール加速度")]
		[Description("スクロール速さの加速度を0～2で設定。\n加速したくない場合は 0 を設定してください。具体的な加速処理は1ステップごとに ScrollStep へ、カウント * ScrollAcceleration を加算していきます。ただし、最大 ScrollStep * 10 で打ち止めです。"), Category("03 : アニメーション設定")]
		public float ScrollAcceleration { set; get; }

		/// <summary>
		/// ショートカットキー有効/無効
		/// </summary>
		[PropertyDisplayName("01 : ショートカットキー有効/無効")]
		[Description("ショートカットキーを有効にする。\ntrue : 有効。\nfalse : 無効"), Category("04 : ショートカットキー設定")]
		public bool EnableShortcutKey { set; get; }
		/// <summary>
		/// 再生/停止 ショートカットキー設定
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("02 : 再生/停止")]
		[Description("再生/停止\n（無効にしたい場合は「None」と設定（入力）してください。）"), Category("04 : ショートカットキー設定")]
		public System.Windows.Forms.Keys PlaySCKey { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string PlaySCKeyAsString
		{
			get { return Generic.ConvertToString(PlaySCKey); }
			set { PlaySCKey = Generic.ConvertFromString<System.Windows.Forms.Keys>(value); }
		}
		/// <summary>
		/// 前の曲 ショートカットキー設定
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("03 : 前の曲")]
		[Description("前の曲\n（無効にしたい場合は「None」と設定（入力）してください。）"), Category("04 : ショートカットキー設定")]
		public System.Windows.Forms.Keys PrevSCKey { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string PrevSCKeyAsString
		{
			get { return Generic.ConvertToString(PrevSCKey); }
			set { PrevSCKey = Generic.ConvertFromString<System.Windows.Forms.Keys>(value); }
		}
		/// <summary>
		/// 次の曲 ショートカットキー設定
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("04 : 次の曲")]
		[Description("次の曲\n（無効にしたい場合は「None」と設定（入力）してください。）"), Category("04 : ショートカットキー設定")]
		public System.Windows.Forms.Keys NextSCKey { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string NextSCKeyAsString
		{
			get { return Generic.ConvertToString(NextSCKey); }
			set { NextSCKey = Generic.ConvertFromString<System.Windows.Forms.Keys>(value); }
		}
		/// <summary>
		/// 音量を上げる ショートカットキー設定
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("05 : 音量を上げる")]
		[Description("音量を下げる\n（無効にしたい場合は「None」と設定（入力）してください。）"), Category("04 : ショートカットキー設定")]
		public System.Windows.Forms.Keys VolumeUpSCKey { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string VolumeUpSCKeyAsString
		{
			get { return Generic.ConvertToString(VolumeUpSCKey); }
			set { VolumeUpSCKey = Generic.ConvertFromString<System.Windows.Forms.Keys>(value); }
		}
		/// <summary>
		/// 音量を下げる ショートカットキー設定
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("06 : 音量を下げる")]
		[Description("音量を上げる\n（無効にしたい場合は「None」と設定（入力）してください。）"), Category("04 : ショートカットキー設定")]
		public System.Windows.Forms.Keys VolumeDownSCKey { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string VolumeDownSCKeyAsString
		{
			get { return Generic.ConvertToString(VolumeDownSCKey); }
			set { VolumeDownSCKey = Generic.ConvertFromString<System.Windows.Forms.Keys>(value); }
		}
		/// <summary>
		/// 通知切り替え ショートカットキー設定
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("07 : 通知")]
		[Description("通知の有効/無効切り替え\n（無効にしたい場合は「None」と設定（入力）してください。）"), Category("04 : ショートカットキー設定")]
		public System.Windows.Forms.Keys SwitchNotifySCKey { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string SwitchNotifySCKeyAsString
		{
			get { return Generic.ConvertToString(SwitchNotifySCKey); }
			set { SwitchNotifySCKey = Generic.ConvertFromString<System.Windows.Forms.Keys>(value); }
		}
		/// <summary>
		/// イントロモード切り替え ショートカットキー設定
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("08 : イントロモード")]
		[Description("イントロモードの有効/無効切り替え\n（無効にしたい場合は「None」と設定（入力）してください。）"), Category("04 : ショートカットキー設定")]
		public System.Windows.Forms.Keys SwitchIntroModeSCKey { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string SwitchIntroModeSCKeyAsString
		{
			get { return Generic.ConvertToString(SwitchIntroModeSCKey); }
			set { SwitchIntroModeSCKey = Generic.ConvertFromString<System.Windows.Forms.Keys>(value); }
		}
		/// <summary>
		/// プレーヤー切り替え ショートカットキー設定
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("09 : プレーヤー切り替え")]
		[Description("プレーヤー切り替え。\n現在使用しているプレーヤーを切り替えます（メニューで他のプレーヤーを選択するのと同じです）。\n（無効にしたい場合は「None」と設定（入力）してください。）"), Category("04 : ショートカットキー設定")]
		public System.Windows.Forms.Keys SwitchUsePlayerSCKey { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string SwitchUsePlayerSCKeyAsString
		{
			get { return Generic.ConvertToString(SwitchUsePlayerSCKey); }
			set { SwitchUsePlayerSCKey = Generic.ConvertFromString<System.Windows.Forms.Keys>(value); }
		}
		/// <summary>
		/// 再通知 ショートカットキー設定
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("10 : 再通知")]
		[Description("再生曲の再通知。\n現在再生されている曲の曲情報を再通知します。\n（無効にしたい場合は「None」と設定（入力）してください。）"), Category("04 : ショートカットキー設定")]
		public System.Windows.Forms.Keys NotiftPlayingTrackInfoSCKey { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string NotiftPlayingTrackInfoSCKeyAsString
		{
			get { return Generic.ConvertToString(SwitchIntroModeSCKey); }
			set { SwitchIntroModeSCKey = Generic.ConvertFromString<System.Windows.Forms.Keys>(value); }
		}
		
		/// <summary>
		/// イントロ再生開始時間[s]
		/// </summary>
		[PropertyDisplayName("01 : イントロ再生開始時間[s]")]
		[Description("イントロ再生開始時間[s]を 0 ～ 1000[s] で設定。"), Category("05 : イントロ設定")]
		public int IntroStartTime { set; get; }
		/// <summary>
		/// イントロ再生時間[s]
		/// </summary>
		[PropertyDisplayName("02 : イントロ再生時間[s]")]
		[Description("イントロ再生時間[s]を 1 ～ 1000[s] で設定。"), Category("05 : イントロ設定")]
		public int IntroPlayTime { set; get; }
		/// <summary>
		/// イントロ再生時、曲の終わりをフェードアウトする
		/// </summary>
		[PropertyDisplayName("03 : フェードアウト")]
		[Description("イントロ再生時、曲の終わりをフェードアウトする"), Category("05 : イントロ設定")]
		public bool EnableIntroFadeout { set; get; }
		/// <summary>
		/// イントロ再生時、曲の開始をフェードインする
		/// </summary>
		[PropertyDisplayName("04 : フェードイン")]
		[Description("イントロ再生時、曲の開始をフェードインする"), Category("05 : イントロ設定")]
		public bool EnableIntroFadein { set; get; }
		/// <summary>
		/// イントロ再生時のフェード時間 100[ms] 間隔で設定。
		/// </summary>
		[PropertyDisplayName("05 : フェード時間[ms]")]
		[Description("イントロ再生時のフェード時間を 100～10000[ms] で 100[ms] 間隔で設定。"), Category("05 : イントロ設定")]
		public int IntroFadeTime { set; get; }
		/// <summary>
		/// イントロ再生時間の単位指定
		/// </summary>
		[PropertyDisplayName("06 : イントロ再生開始時間、再生時間の単位")]
		[Description("イントロ再生開始時間、再生時間の単位を設定。[%]の場合、総演奏時間に対するパーセンテージをなります。設定値が100%以上こえた場合は通常再生となります。"), Category("05 : イントロ設定")]
		public IntroTimeUnit IntroTimeUnit { set; get; }

		/// <summary>
		/// 起動時通知を無効にする
		/// </summary>
		[PropertyDisplayName("01 : 起動時通知を無効にする")]
		[Description("起動時通知を無効にする。\ntrue : 起動時に通知無効。\nfalse : 起動時に通知有効"), Category("06 : その他設定")]
		public bool InitDisableNotify { set; get; }
		/// <summary>
		/// 使用プレーヤ
		/// </summary>
		[PropertyDisplayName("02 : 使用プレーヤ")]
		[Description("監視対象のプレーヤを選択。"), Category("06 : その他設定")]
		public PlayerType PlayerType { set; get; }
		/// <summary>
		/// タスクトレイのバルーンチップの表示を行うかの設定
		/// </summary>
		[PropertyDisplayName("03 : バルーンチップの表示設定")]
		[Description("バルーンチップの表示を行うかの設定\ntrue : 行う、false : 行わない。\n（注意！ バルーン表示を行うと、キャプチャソフトが起動している場合表示がもたつきます。）"), Category("06 : その他設定")]
		public bool EnableBalloonTip { set; get; }
		/// <summary>
		/// タスクトレイのバルーンチップの表示時間[ms]
		/// </summary>
		[PropertyDisplayName("04 : バルーンチップ表示時間[ms]")]
		[Description("バルーンチップの表示時間[ms]\n（必ずしもこの時間で表示/消去されるわけではないことに注意してください。ユーザーのキーボード、マウス操作がない場合は消えません。これはWinamowsの仕様です。）"), Category("06 : その他設定")]
		public int BalloonTipTimeout { set; get; }
		/// <summary>
		/// プレーヤーサーチの優先プレイヤー
		/// </summary>
		[PropertyDisplayName("05 : プレーヤーサーチの優先プレイヤー")]
		[Description("プレーヤーサーチの優先プレイヤー\nプレーヤーサーチが有効な場合の優先順位。"), Category("06 : その他設定")]
		public PlayerSearchPriority PlayerSearchPriority { set; get; }
		/// <summary>
		/// プレーヤーサーチを行うかの設定
		/// </summary>
		[PropertyDisplayName("06 : プレーヤーサーチ")]
		[Description("プレーヤーサーチを行うかの設定\ntrue : 行う、false : 行わない。"), Category("06 : その他設定")]
		public bool EnablePlayerSearch { set; get; }
		/// <summary>
		/// 優先プレーヤーが起動した場合にモニタープレーヤを強制で切り替えるかの設定。
		/// </summary>
		[PropertyDisplayName("07 : 強制プレイヤー変更")]
		[Description("優先プレーヤーが起動した場合にモニタープレーヤを強制で切り替えるかの設定。\ntrue : 行う、false : 行わない。"), Category("06 : その他設定")]
		public bool EnableForcedChangePlayer { set; get; }

		/// <summary>
		/// ヘルプ URL
		/// </summary>
		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string HelpURL { set; get; }

		/// <summary>
		/// プラグインイベント監視タイマーインターバル
		/// </summary>
		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public int PluginEventInterval { set; get; }

		/// <summary>
		/// 標準のコンストラクタ
		/// </summary>
		public AppSetting()
		{
			//TopMost = false;
			BoardLocation = new Point(100, 0);
			NoneArtworkImageFilename = "Noimage.png";
			DispNoneArtworkImage = true;
			iTunesBoardLocation = new Point(100, 0);
			WinampBoardLocation = new Point(100, 0);
			EnablePlayerBoardLocation = false;

			TextFormat = "%A %L%N%I - %T";
			TextFont = new Font("ＭＳ ゴシック", 36, FontStyle.Bold);
			TextColor1 = Color.FromArgb(255, 255, 64);
			TextColor2 = Color.FromArgb(64, 64, 0);
			TextDrawingMode = NauPlaying.TextDrawingMode.Border;
			OutlineWidth = 2;
			UseSkinTextDrawingSetting = false;
			UseSkinTextFont = false;
			ZoomMaginification = 1.0f;
			BoardAlpah = 0.85f;

			FadeinTime = 250;
			FadeoutTime = 250;
			ScrollMode = ScrollMode.Mode2;
			ScrollStep = 5;
			ScrollStopTime = 300;
			NoneScrollStopTime = 1750;
			StepTime = 50;
			ScrollAcceleration = 1;

			EnableBalloonTip = true;
			BalloonTipTimeout = 100;
			InitDisableNotify = false;
			PlayerType = NauPlaying.PlayerType.iTunes;

			EnableShortcutKey = false;
			PlaySCKey = System.Windows.Forms.Keys.Control | System.Windows.Forms.Keys.Alt | System.Windows.Forms.Keys.Space;
			PrevSCKey = System.Windows.Forms.Keys.Control | System.Windows.Forms.Keys.Alt | System.Windows.Forms.Keys.Left;
			NextSCKey = System.Windows.Forms.Keys.Control | System.Windows.Forms.Keys.Alt | System.Windows.Forms.Keys.Right;
			VolumeUpSCKey = System.Windows.Forms.Keys.Control | System.Windows.Forms.Keys.Alt | System.Windows.Forms.Keys.Up;
			VolumeDownSCKey = System.Windows.Forms.Keys.Control | System.Windows.Forms.Keys.Alt | System.Windows.Forms.Keys.Down;
			SwitchNotifySCKey = System.Windows.Forms.Keys.Control | System.Windows.Forms.Keys.Alt | System.Windows.Forms.Keys.N;
			SwitchIntroModeSCKey = System.Windows.Forms.Keys.Control | System.Windows.Forms.Keys.Alt | System.Windows.Forms.Keys.I;
			SwitchUsePlayerSCKey = System.Windows.Forms.Keys.Control | System.Windows.Forms.Keys.Alt | System.Windows.Forms.Keys.S;
			NotiftPlayingTrackInfoSCKey = System.Windows.Forms.Keys.Control | System.Windows.Forms.Keys.Alt | System.Windows.Forms.Keys.T;

			IntroStartTime = 0;
			IntroPlayTime = 90;
			EnableIntroFadein = true;
			EnableIntroFadeout = true;
			IntroFadeTime = 500;
			IntroTimeUnit = NauPlaying.IntroTimeUnit.Sec;
			PlayerSearchPriority = NauPlaying.PlayerSearchPriority.iTunes;
			EnablePlayerSearch = false;
			EnableForcedChangePlayer = false;

			HelpURL = "http://www.maroon.dti.ne.jp/jyaku9/nau_help/index.html";
			PluginEventInterval = 100;
		}

		/// <summary>
		/// シリアライズ
		/// </summary>
		/// <param name="filename">xlmファイル名</param>
		public void Serialize(string filename)
		{
			System.Xml.Serialization.XmlSerializer serializer = new System.Xml.Serialization.XmlSerializer(typeof(AppSetting));

			using(System.IO.FileStream fs = new System.IO.FileStream(filename, System.IO.FileMode.Create))
			{
				serializer.Serialize(fs, this);
			}
		}

		/// <summary>
		/// デシリアライズ
		/// </summary>
		/// <param name="filename">xlmファイル名</param>
		/// <returns></returns>
		public static AppSetting Deserialize(string filename)
		{
			System.Xml.Serialization.XmlSerializer serializer = new System.Xml.Serialization.XmlSerializer(typeof(AppSetting));

			using(System.IO.FileStream fs = new System.IO.FileStream(filename, System.IO.FileMode.Open))
			{
				AppSetting setting = (AppSetting)serializer.Deserialize(fs);
				return setting;
			}
		}

		/// <summary>
		/// ディープコピー
		/// </summary>
		/// <returns></returns>
		public AppSetting DeepCopy()
		{
			AppSetting obj = (AppSetting)MemberwiseClone();

			obj.TextFont = (Font)TextFont.Clone();

			return obj;
		}
	}
}
