﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace NauPlaying
{
	/// <summary>
	/// フォント選択コントロールクラス
	/// </summary>
	public partial class FontSelectBox : UserControl
	{
		/// <summary>
		/// 表示テキストタイプ
		/// </summary>
		public enum DisplayStringTypes
		{
			/// <summary>
			/// Text プロパティーを表示
			/// </summary>
			Text,
			/// <summary>
			/// フォント名を使用する
			/// </summary>
			FontName,
		}

		/// <summary>
		/// 表示テキストタイプの設定、取得
		/// </summary>
		public DisplayStringTypes DisplayStringType { set; get; }

		/// <summary>
		/// 標準のコンストラクタ
		/// </summary>
		public FontSelectBox()
		{
			Text = "abcABCあいう阿伊宇";
			InitializeComponent();
			DisplayStringType = DisplayStringTypes.FontName;
		}

		/// <summary>
		/// マウスイベント：クリック
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void FontSelectBox_MouseClick(object sender, MouseEventArgs e)
		{
			if(e.Button == System.Windows.Forms.MouseButtons.Left)
			{
				FontDialog dlg = new FontDialog();

				dlg.Font = Font;

				if(dlg.ShowDialog() == DialogResult.OK)
				{
					Font = dlg.Font;
				}
			}
		}

		/// <summary>
		/// マウスイベント：移動
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void FontSelectBox_MouseMove(object sender, MouseEventArgs e)
		{
			// カーソルの変更
			Cursor = Cursors.Hand;
		}

		/// <summary>
		/// 描画イベント
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void FontSelectBox_Paint(object sender, PaintEventArgs e)
		{
			Graphics g = e.Graphics;

			string str = Font.Name;
			if(DisplayStringType == DisplayStringTypes.Text)
				str = Text;

			using(Brush backBrush = new SolidBrush(BackColor))
			using(Brush foreBrush = new SolidBrush(ForeColor))
			{
				SizeF size = g.MeasureString(str, Font);
				int offset = (int)(((float)ClientSize.Height - size.Height) / 2f);

				g.FillRectangle(backBrush, new Rectangle(0, 0, ClientSize.Width, ClientSize.Height));
				g.DrawString(str, Font, foreBrush, new PointF(1, offset));
			}
		}
	}
}
