﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;

namespace NauPlaying
{
	/// <summary>
	/// 設定変更ダイアログ
	/// </summary>
	public partial class FormConfig : Form
	{
		/// <summary>
		/// バインドされるデータクラス
		/// </summary>
		public class NotifyPropertyChangedData : INotifyPropertyChanged
		{
			public event PropertyChangedEventHandler PropertyChanged;
			
			/// <summary>
			/// プロパティ変更通知
			/// </summary>
			/// <param name="info"></param>
			private void NotifyPropertyChanged(String info)
			{
				if(PropertyChanged != null)
				{
					PropertyChanged(this, new PropertyChangedEventArgs(info));
				}
			}

			/// <summary>
			/// ボードアルファ
			/// </summary>
			private int _boardAlpha;
			/// <summary>
			/// ボードアルファの取得、設定
			/// </summary>
			public int BoardAlpha
			{
				get { return _boardAlpha; }
				set
				{
					if(value != _boardAlpha)
					{
						_boardAlpha = value;
						NotifyPropertyChanged("Value");
					}
				}
			}

			/// <summary>
			/// フェードイン時間[ms]
			/// </summary>
			private int _fadeinTime;
			/// <summary>
			/// フェードイン時間[ms]の取得、設定
			/// </summary>
			public int FadeinTime
			{
				get { return _fadeinTime; }
				set
				{
					if(value != _fadeinTime)
					{
						_fadeinTime = value;
						NotifyPropertyChanged("Value");
					}
				}
			}

			/// <summary>
			/// フェードアウト時間[ms]
			/// </summary>
			private int _fadeoutTime;
			/// <summary>
			/// フェードアウト時間[ms]の取得、設定
			/// </summary>
			public int FadeoutTime
			{
				get { return _fadeoutTime; }
				set
				{
					if(value != _fadeoutTime)
					{
						_fadeoutTime = value;
						NotifyPropertyChanged("Value");
					}
				}
			}

			/// <summary>
			/// スクルロール停止時間[ms]
			/// </summary>
			private int _scrollStopTime;
			/// <summary>
			/// スクルロール停止時間[ms]の取得、設定
			/// </summary>
			public int ScrollStopTime
			{
				get { return _scrollStopTime; }
				set
				{
					if(value != _scrollStopTime)
					{
						_scrollStopTime = value;
						NotifyPropertyChanged("Value");
					}
				}
			}

			/// <summary>
			/// スクルロールステップ時間[ms]
			/// </summary>
			private int _scrollStepTime;
			/// <summary>
			/// スクルロールステップ時間[ms]の取得、設定
			/// </summary>
			public int ScrollStepTime
			{
				get { return _scrollStepTime; }
				set
				{
					if(value != _scrollStepTime)
					{
						_scrollStepTime = value;
						NotifyPropertyChanged("Value");
					}
				}
			}

			/// <summary>
			/// スクルロール移動量[px]
			/// </summary>
			private int _scrollStep;
			/// <summary>
			/// スクルロール移動量[px]の取得、設定
			/// </summary>
			public int ScrollStep
			{
				get { return _scrollStep; }
				set
				{
					if(value != _scrollStep)
					{
						_scrollStep = value;
						NotifyPropertyChanged("Value");
					}
				}
			}

			/// <summary>
			/// イントロ開始時間[s]
			/// </summary>
			private int _introStartTime;
			/// <summary>
			/// イントロ開始時間[s]
			/// </summary>
			public int IntroStartTime
			{
				get { return _introStartTime; }
				set
				{
					if(value != _introStartTime)
					{
						_introStartTime = value;
						NotifyPropertyChanged("Value");
					}
				}
			}

			/// <summary>
			/// イントロ再生時間[s]
			/// </summary>
			private int _introPlayTime;
			/// <summary>
			/// イントロ再生時間[s]
			/// </summary>
			public int IntroPlayTime
			{
				get { return _introPlayTime; }
				set
				{
					if(value != _introPlayTime)
					{
						_introPlayTime = value;
						NotifyPropertyChanged("Value");
					}
				}
			}

			/// <summary>
			/// イントロフェード時間[ms]
			/// </summary>
			private int _introFadeTime;
			/// <summary>
			/// イントロフェード時間[ms]
			/// </summary>
			public int IntroFadeTime
			{
				get { return _introFadeTime; }
				set
				{
					if(value != _introFadeTime)
					{
						_introFadeTime = value;
						NotifyPropertyChanged("Value");
					}
				}
			}
		}

		/// <summary>
		/// ラジオボタンで選択されているスクロールモードの取得、設定
		/// </summary>
		private ScrollMode SelectedRadioButton_ScrollMode
		{
			get
			{
				if(_radioButton_ScrollMode2.Checked) return ScrollMode.Mode1;
				if(_radioButton_ScrollMode3.Checked) return ScrollMode.Mode2;
				return ScrollMode.None;
			}
			set
			{
				switch(value)
				{
				case ScrollMode.None:
					_radioButton_ScrollMode1.Checked = true;
					break;
				case ScrollMode.Mode1:
					_radioButton_ScrollMode2.Checked = true;
					break;
				case ScrollMode.Mode2:
					_radioButton_ScrollMode3.Checked = true;
					break;
				}
			}
		}

		/// <summary>
		/// ラジオボタンで選択されているテキスト描画モードの取得、設定
		/// </summary>
		private TextDrawingMode SelectedRadioButton_TextDrawingMode
		{
			get
			{
				if(_radioButton_TextDrawingMode2.Checked) return TextDrawingMode.Outline;
				if(_radioButton_TextDrawingMode3.Checked) return TextDrawingMode.Border;
				if(_radioButton_TextDrawingMode4.Checked) return TextDrawingMode.Shadow;
				if(_radioButton_TextDrawingMode5.Checked) return TextDrawingMode.BorderAndShadow;
				return TextDrawingMode.Normal;
			}
			set
			{
				switch(value)
				{
				case TextDrawingMode.Normal:
					_radioButton_TextDrawingMode1.Checked = true;
					break;
				case TextDrawingMode.Outline:
					_radioButton_TextDrawingMode2.Checked = true;
					break;
				case TextDrawingMode.Border:
					_radioButton_TextDrawingMode3.Checked = true;
					break;
				case TextDrawingMode.Shadow:
					_radioButton_TextDrawingMode4.Checked = true;
					break;
				case TextDrawingMode.BorderAndShadow:
					_radioButton_TextDrawingMode5.Checked = true;
					break;
				}
			}
		}

		/// <summary>
		/// ボードフォーム
		/// </summary>
		FormBoard _formBoard;

		/// <summary>
		/// アプリケーション設定
		/// </summary>
		public AppSetting AppSetting { set; get; }
		/// <summary>
		/// スキン設定
		/// </summary>
		public Skin.SkinSetting SkinSetting { set; get; }

		/// <summary>
		/// 選択されているスキンを取得。
		/// 選択されていない場合は null
		/// </summary>
		/// <returns></returns>
		private Skin.SkinInfo SelectedSkinSetting
		{
			get
			{
				if(_skinManager.Count == 0)
					return null;

				int idx = _listBox_Skins.SelectedIndex;
				if(idx < 0 || idx >= _skinManager.Count)
					return null;

				return _skinManager[idx];
			}
		}

		/// <summary>
		/// 選択されているイントロ時間単位の取得、設定
		/// </summary>
		private IntroTimeUnit SelectedIntroTimeUnit
		{
			set
			{
				if(value == IntroTimeUnit.Sec)
				{
					_radioButton_IntroUnitoMode1.Checked = true;
					_radioButton_IntroUnitoMode2.Checked = false;
				}
				else
				{
					_radioButton_IntroUnitoMode2.Checked = false;
					_radioButton_IntroUnitoMode2.Checked = true;
				}
			}
			get
			{
				if(_radioButton_IntroUnitoMode1.Checked)
					return IntroTimeUnit.Sec;
				return IntroTimeUnit.Percentage;
			}
		}

		/// <summary>
		/// スキンマネージャ
		/// </summary>
		Skin.SkinManager _skinManager;

		/// <summary>
		/// コンストラクタ
		/// </summary>
		public FormConfig()
		{
			_formBoard = new FormBoard();

			_skinManager = new Skin.SkinManager();

			InitializeComponent();
		}

		/// <summary>
		/// フォームが表示されるときに呼ばれる
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void FormConfig_Load(object sender, EventArgs e)
		{
			_propertyGrid_AppSetting.SelectedObject = AppSetting;
			_propertyGrid_SkinSetting.SelectedObject = SkinSetting;

			UpdateSkinList();

			_bindingSource.DataSource = new NotifyPropertyChangedData();

			UpdateAppDataFromGenericTab(false);
		}

		/// <summary>
		/// フォームが閉じる前に呼ばれる
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void FormConfig_FormClosing(object sender, FormClosingEventArgs e)
		{
			if(_formBoard != null)
			{
				if(_formBoard.IsAnimation)
					_formBoard.StopAnimation();

				_formBoard.Close();
				_formBoard.Dispose();
				_formBoard = null;
			}

			if(DialogResult == System.Windows.Forms.DialogResult.OK)
				UpdateAppDataFromGenericTab(true);
		}

		/// <summary>
		/// 一般設定のタブの設定データを AppSetting へ設定、取得する
		/// </summary>
		/// <param name="update">true で AppSetting へ設定。false で AppSetting をタブの設定データに設定。</param>
		private void UpdateAppDataFromGenericTab(bool update)
		{
			NotifyPropertyChangedData data = (NotifyPropertyChangedData)_bindingSource.DataSource;

			if(update)
			{
				AppSetting.TextColor1 = _colorSelectBox_TextColor1.Color;
				AppSetting.TextColor2 = _colorSelectBox_TextColor2.Color;
				AppSetting.TextFont = _fontSelectBox_TextFont.Font;
				AppSetting.BoardAlpah = (float)((decimal)data.BoardAlpha / 100m);
				AppSetting.FadeinTime = data.FadeinTime;
				AppSetting.FadeoutTime = data.FadeoutTime;
				AppSetting.NoneScrollStopTime = data.ScrollStopTime;
				AppSetting.ScrollStopTime = data.ScrollStepTime;
				AppSetting.ScrollStep = data.ScrollStep;
				AppSetting.ScrollMode = SelectedRadioButton_ScrollMode;
				AppSetting.TextDrawingMode = SelectedRadioButton_TextDrawingMode;
				AppSetting.OutlineWidth = (int)_numericUpDown_OutlineWidth.Value;

				AppSetting.EnableIntroFadein = _checkBox_EnableIntroFadein.Checked;
				AppSetting.EnableIntroFadeout = _checkBox_EnableIntroFadeout.Checked;
				AppSetting.IntroStartTime = data.IntroStartTime;
				AppSetting.IntroPlayTime = data.IntroPlayTime;
				AppSetting.IntroFadeTime = data.IntroFadeTime;
				AppSetting.IntroTimeUnit = SelectedIntroTimeUnit;
			}
			else
			{
				_colorSelectBox_TextColor1.Color = AppSetting.TextColor1;
				_colorSelectBox_TextColor2.Color = AppSetting.TextColor2;
				_fontSelectBox_TextFont.Font = AppSetting.TextFont;
				data.BoardAlpha = (int)((decimal)AppSetting.BoardAlpah * 100m);
				data.FadeinTime = AppSetting.FadeinTime;
				data.FadeoutTime = AppSetting.FadeoutTime;
				data.ScrollStopTime = AppSetting.NoneScrollStopTime;
				data.ScrollStepTime = AppSetting.ScrollStopTime;
				data.ScrollStep = AppSetting.ScrollStep;
				SelectedRadioButton_ScrollMode = AppSetting.ScrollMode;
				SelectedRadioButton_TextDrawingMode = AppSetting.TextDrawingMode;
				_numericUpDown_OutlineWidth.Value = AppSetting.OutlineWidth;

				_checkBox_EnableIntroFadein.Checked = AppSetting.EnableIntroFadein;
				_checkBox_EnableIntroFadeout.Checked = AppSetting.EnableIntroFadeout;
				data.IntroStartTime = AppSetting.IntroStartTime;
				data.IntroPlayTime = AppSetting.IntroPlayTime;
				data.IntroFadeTime = AppSetting.IntroFadeTime;
				SelectedIntroTimeUnit = AppSetting.IntroTimeUnit;
			}
		}

		/// <summary>
		/// テスト表示
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void _button_Apply_Click(object sender, EventArgs e)
		{
			// 簡易設定タブの場合はテスト前に AppSetting へデータを反映させる
			if(_tabControl.SelectedIndex == 0)
				UpdateAppDataFromGenericTab(true);

			if(_formBoard.IsAnimation)
				_formBoard.StopAnimation();

			PluginBase.TrackInfo info = new PluginBase.TrackInfo();
			info.Album = "アルバム名";
			info.Artist = "アーティスト名";
			info.Name = "トラック名";
			info.TrackNo = "0";

			_formBoard.InitializeBoard(AppSetting, SkinSetting, new BoardData(info.Format(AppSetting.TextFormat), Skin.SkinRender.NoneArtworkImage));

			if(!_formBoard.Visible)
				_formBoard.ShowWithoutActiveTopMost(true);

			_formBoard.StartAnimation();
		}

		/// <summary>
		/// タブの切り替えイベント
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void _tabControl_SelectedIndexChanged(object sender, EventArgs e)
		{
			if(_tabControl.SelectedIndex == 0 || _tabControl.SelectedIndex == 1 || _tabControl.SelectedIndex == 2)
			{
				editLocationToolStripMenuItem.Enabled = true;

				if(_tabControl.SelectedIndex == 0)
					UpdateAppDataFromGenericTab(false);
				else if(_tabControl.SelectedIndex == 2)
				{
					UpdateAppDataFromGenericTab(true);
					_propertyGrid_AppSetting.SelectedObject = null;
					_propertyGrid_AppSetting.SelectedObject = AppSetting;
				}
			}
			else
				editLocationToolStripMenuItem.Enabled = false;

			if(_tabControl.SelectedIndex == 3)
				editRegionToolStripMenuItem.Enabled = true;
			else
				editRegionToolStripMenuItem.Enabled = false;

			if(_tabControl.SelectedIndex == 1)
			{
				skinToolStripMenuItem.Enabled = true;

				if(_listBox_Skins.SelectedIndex < 0 && _listBox_Skins.Items.Count > 0)
					_listBox_Skins.SelectedIndex = 0;
			}
			else
				skinToolStripMenuItem.Enabled = false;
		}

		/// <summary>
		/// リストボックスのダブルクリックでスキンを設定
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void _listBox_Skins_DoubleClick(object sender, EventArgs e)
		{
			Skin.SkinInfo selectedInfo = SelectedSkinSetting;
			if(selectedInfo == null)
				return;

			if(MessageBox.Show(this, "現在のスキン設定を「" + selectedInfo.SkinSetting.SkinTitle + "」で上書きしますがよろしいですか？", Generic.AppTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) != System.Windows.Forms.DialogResult.Yes)
				return;

			// イメージのコピー
			try
			{
				string srcImagePath = Path.GetDirectoryName(selectedInfo.Fullpath) + "\\" + Generic.BkImageFilename;
				string dstImagePath = Generic.BkImageFilepath;

				File.Copy(srcImagePath, dstImagePath, true);
			}
			catch(Exception)
			{
				MessageBox.Show(this, "バッググランドイメージのコピーに失敗しました。\n手動でのコピーを行ってください。", Generic.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1);
			}
			// 設定の上書き
			SkinSetting = selectedInfo.SkinSetting.DeepCopy();
			_propertyGrid_SkinSetting.SelectedObject = SkinSetting;
		}

		/// <summary>
		/// リストボックスの選択でスキン情報の表示を行う
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void _listBox_Skins_SelectedIndexChanged(object sender, EventArgs e)
		{
			// ピクチャボックス Image に有効なビットマップがあった場合 Dispose してからイメージをセットする
			Action<Bitmap> _pictureBox_SkinBkImage_DisposingSet = (bmp) =>
				{
					if(_pictureBox_SkinBkImage.Image != null)
					{
						Bitmap old = (Bitmap)_pictureBox_SkinBkImage.Image;
						_pictureBox_SkinBkImage.Image = null;
						old.Dispose();
					}

					_pictureBox_SkinBkImage.Image = bmp;
				};

			Skin.SkinInfo selectedInfo = SelectedSkinSetting;
			if(selectedInfo == null)
				return;

			Skin.SkinSetting set = selectedInfo.SkinSetting;

			_label_SkinTitle.Text = "タイトル：" + set.SkinTitle;
			_label_SkinCopyright.Text = "著作権：" + set.Copyright;
			_label_SkinComment.Text = "コメント：" + set.Comment;
			_label_SkinVersion.Text = "バージョン：" + set.Version;
			_label_SkinFilepath.Text = "ファイルパス：" + selectedInfo.Fullpath;

			// イメージの取得
			if(set.EnableArtwork)
			{
				try
				{
					using(Bitmap srcBmp = new Bitmap(Path.GetDirectoryName(selectedInfo.Fullpath) + "\\" + Generic.BkImageFilename))
					{
						Bitmap dstBmp = new Bitmap(_pictureBox_SkinBkImage.Size.Width, _pictureBox_SkinBkImage.Size.Height, srcBmp.PixelFormat);

						using(Graphics g = Graphics.FromImage(dstBmp))
						{
							// 正方ではない場合アスペクトを維持する
							Rectangle dest;

							if(srcBmp.Width != srcBmp.Height)
							{
								if(srcBmp.Width > srcBmp.Height)
								{
									float ratio = (float)srcBmp.Height / (float)srcBmp.Width;
									int height = (int)((float)dstBmp.Size.Height * ratio);

									int offset = (dstBmp.Size.Width - height) / 2;

									dest = new Rectangle(0, offset, dstBmp.Size.Width, height);
								}
								else
								{
									float ratio = (float)srcBmp.Width / (float)srcBmp.Height;
									int width = (int)((float)dstBmp.Size.Width * ratio);

									int offset = (dstBmp.Size.Height - width) / 2;

									dest = new Rectangle(offset, 0, width, dstBmp.Size.Height);
								}
							}
							else
								dest = new Rectangle(0, 0, dstBmp.Size.Width, dstBmp.Size.Height);

							g.DrawImage(srcBmp, dest);
						}

						_pictureBox_SkinBkImage_DisposingSet(dstBmp);
					}
				}
				catch(Exception)
				{
					_pictureBox_SkinBkImage_DisposingSet(null);
				}
			}
			else
			{
				_pictureBox_SkinBkImage_DisposingSet(null);
			}
		}

		/// <summary>
		/// アプリケーション設定プロパティの変更イベント アプリケーション設定の検証
		/// </summary>
		/// <param name="s"></param>
		/// <param name="e"></param>
		private void _propertyGrid_AppSetting_PropertyValueChanged(object s, PropertyValueChangedEventArgs e)
		{
			string name = e.ChangedItem.PropertyDescriptor.Name;

			switch(name)
			{
			case "BoardLocation":
				AppSetting.BoardLocation = Generic.ModifyingPoint((Point)e.ChangedItem.Value, 0, 10000, 0, 10000);
				break;

			case "BoardAlpah":
				AppSetting.BoardAlpah = Generic.ModifyinFloatValue((float)e.ChangedItem.Value, 0, 1);
				break;

			case "OutlineWidth":
				AppSetting.OutlineWidth = Generic.ModifyinIntValue((int)e.ChangedItem.Value, 1, 10);
				break;

			case "FadeinTime":
				AppSetting.FadeinTime = Generic.ModifyinIntValue((int)e.ChangedItem.Value, 0, 10000);
				break;

			case "FadeoutTime":
				AppSetting.FadeoutTime = Generic.ModifyinIntValue((int)e.ChangedItem.Value, 0, 10000);
				break;

			case "ScrollStep":
				AppSetting.ScrollStep = Generic.ModifyinIntValue((int)e.ChangedItem.Value, 1, 100);
				break;

			case "ScrollStopTime":
				AppSetting.ScrollStopTime = Generic.ModifyinIntValue((int)e.ChangedItem.Value, 0, 10000);
				break;

			case "NoneScrollStopTime":
				AppSetting.NoneScrollStopTime = Generic.ModifyinIntValue((int)e.ChangedItem.Value, 0, 10000);
				break;

			case "StepTime":
				AppSetting.StepTime = Generic.ModifyinIntValue((int)e.ChangedItem.Value, 10, 1000);
				break;

			case "ScrollAcceleration":
				AppSetting.ScrollAcceleration = Generic.ModifyinFloatValue((float)e.ChangedItem.Value, 0, 2);
				break;

			case "IntroStartTime":
				AppSetting.IntroStartTime = Generic.ModifyinIntValue((int)e.ChangedItem.Value, 0, 1000);
				break;

			case "IntroPlayTime":
				AppSetting.IntroPlayTime = Generic.ModifyinIntValue((int)e.ChangedItem.Value, 1, 1000);
				break;

			case "IntroFadeTime":
				AppSetting.IntroFadeTime = Generic.ModifyinIntValue((int)e.ChangedItem.Value, 100, 10000);
				break;
			}
		}

		/// <summary>
		/// スキン設定プロパティの変更イベント スキン設定の検証
		/// </summary>
		/// <param name="s"></param>
		/// <param name="e"></param>
		private void _propertyGrid_SkinSetting_PropertyValueChanged(object s, PropertyValueChangedEventArgs e)
		{
			string name = e.ChangedItem.PropertyDescriptor.Name;

			switch(name)
			{
			case "BoardSize":
				SkinSetting.BoardSize = Generic.ModifyingSize((Size)e.ChangedItem.Value, 100, 10000, 100, 10000);
				break;

			case "ArtworkDispRect":
				SkinSetting.ArtworkDispRect = Generic.ModifyingRect((Rectangle)e.ChangedItem.Value, 0, 10000, 0, 10000, 16, 10000, 16, 10000, true);
				break;

			case "TextDispRect":
				SkinSetting.TextDispRect = Generic.ModifyingRect((Rectangle)e.ChangedItem.Value, 0, 10000, 0, 10000, 16, 10000, 16, 10000, false);
				break;

			case "GraduationFactor":
				SkinSetting.GraduationFactor = Generic.ModifyinFloatValue((float)e.ChangedItem.Value, 0, 10);
				break;

			case "BgImageDispRect":
				SkinSetting.BgImageDispRect = Generic.ModifyingRect((Rectangle)e.ChangedItem.Value, 0, 10000, 0, 10000, 16, 10000, 16, 10000, false);
				break;
			}
		}

		/// <summary>
		/// メニュー　現在のスキンを保存
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			// ディレクトリの取得
			FormInputSkinFoldername form = new FormInputSkinFoldername();
			if(form.ShowDialog() != System.Windows.Forms.DialogResult.OK)
				return;

			string dstFolderpath = form.SkinFolderpath;

			// フォルダ生成
			try
			{
				Directory.CreateDirectory(dstFolderpath);
			}
			catch(Exception)
			{
				MessageBox.Show(this, "フォルダの生成に失敗しました。", Generic.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				return;
			}

			Skin.SkinSetting setting = SkinSetting.DeepCopy();

			// イメージのコピー
			if(setting.EnableBgImage)
			{
				try
				{
					string srcImagePath = Generic.BkImageFilepath;
					string dstImagePath = dstFolderpath + "\\" + Generic.BkImageFilename;

					File.Copy(srcImagePath, dstImagePath, true);
				}
				catch(Exception)
				{
					MessageBox.Show(this, "バッググランドイメージのコピーに失敗しました。\n手動でのコピーを行ってください。", Generic.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1);
				}
			}

			// スキン設定の保存
			try
			{
				string settinFilepath = dstFolderpath + "\\" + Generic.SkinSettingFilename;

				setting.Serialize(settinFilepath);
			}
			catch(Exception)
			{
				MessageBox.Show(this, "スキン設定の保存に失敗しました。", Generic.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				return;
			}

			UpdateSkinList();
		}

		/// <summary>
		/// メニューテンプレートスキンの作成
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void createTempSkinToolStripMenuItem_Click(object sender, EventArgs e)
		{
			// ディレクトリの取得
			FormInputSkinFoldername form = new FormInputSkinFoldername();

			form.Message = "テンプレートスキンを作成するフォルダ名を入力してね。";
			form.Title = "テンプレートスキンの作成";

			if(form.ShowDialog() != System.Windows.Forms.DialogResult.OK)
				return;

			string dstFolderpath = form.SkinFolderpath;

			// フォルダ生成
			try
			{
				Directory.CreateDirectory(dstFolderpath);
			}
			catch(Exception)
			{
				MessageBox.Show(this, "フォルダの生成に失敗しました。", Generic.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				return;
			}

			// テンプレートスキン設定の保存
			try
			{
				Skin.SkinSetting setting = new Skin.SkinSetting();
				string settinFilepath = dstFolderpath + "\\" + Generic.SkinSettingFilename;

				setting.Serialize(settinFilepath);
			}
			catch(Exception)
			{
				MessageBox.Show(this, "スキン設定の保存に失敗しました。", Generic.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				return;
			}

			// テンプレートスキンのひな形イメージのコピー
			try
			{
				using(Bitmap artwork = Skin.SkinRender.CreateNoneArtworkImage(new Size(128, 128)))
				{
					artwork.Save(dstFolderpath + "\\" + Generic.BkImageFilename);
				}
			}
			catch(Exception)
			{
				MessageBox.Show(this, "バッググランドイメージの生成に失敗しました。", Generic.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1);
			}

			UpdateSkinList();
		}

		/// <summary>
		/// メニュー スキンの削除
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
		{
			try
			{
				Skin.SkinInfo selectedInfo = SelectedSkinSetting;
				if(selectedInfo == null)
					return;

				string msg = "フォルダ [Skin\\" + Generic.GetLastFolderName(Path.GetDirectoryName(selectedInfo.Fullpath)) + "] 内にある\nスキン[" + selectedInfo.SkinSetting.SkinTitle + "] を\n削除してもよいですか？\nフォルダ内のファイルすべて削除されます。";

				if(MessageBox.Show(this, msg, Generic.AppTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) != System.Windows.Forms.DialogResult.Yes)
					return;

				string folderpath = Path.GetDirectoryName(selectedInfo.Fullpath);
				Directory.Delete(folderpath, true);

				UpdateSkinList();
			}
			catch(Exception ex)
			{
				MessageBox.Show(this, "スキンの削除に失敗しました。\n" + ex.Message, Generic.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				return;
			}
		}

		/// <summary>
		/// メニュー：フォルダのエクスプローラ表示
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void viewFolderToolStripMenuItem_Click(object sender, EventArgs e)
		{
			try
			{
				Skin.SkinInfo selectedInfo = SelectedSkinSetting;
				if(selectedInfo == null)
					return;

				string folderpath = Path.GetDirectoryName(selectedInfo.Fullpath);
				System.Diagnostics.Process.Start(folderpath);
			}
			catch(Exception)
			{
				return;
			}
		}

		/// <summary>
		/// メニュー：領域編集
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void editRegionToolStripMenuItem_Click(object sender, EventArgs e)
		{
			FormRegionEditor form = new FormRegionEditor();

			Skin.SkinSetting sset = SkinSetting;

			form.BoardSize = sset.BoardSize;
			form.ArtworkDispRect = sset.ArtworkDispRect;
			form.TextDispRect = sset.TextDispRect;
			form.BkImageDispRect = sset.BgImageDispRect;

			if(sset.EnableBgImage)
			{
				try
				{
					Bitmap bkImage = new Bitmap(Generic.BkImageFilepath);
					form.BkImage = bkImage;
				}
				catch(Exception ex)
				{
					Generic.UpdateLogFile(ex);
					form.BkImage = null;
					sset.EnableBgImage = false;
				}
			}
			else
			{
				form.BkImage = null;
			}

			if(form.ShowDialog() == System.Windows.Forms.DialogResult.OK)
			{
				sset.ArtworkDispRect = form.ArtworkDispRect;
				sset.TextDispRect = form.TextDispRect;
				sset.BgImageDispRect = form.BkImageDispRect;
				sset.BoardSize = form.BoardSize;

				_propertyGrid_SkinSetting.SelectedObject = sset;
			}

			if(form.BkImage != null)
				form.BkImage.Dispose();
		}

		/// <summary>
		/// メニュー：ウインドウ位置編集
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void editLocationToolStripMenuItem_Click(object sender, EventArgs e)
		{
			FormBoard form = new FormBoard();
			form.TopMost = true;
			form.IsEnableMouseEvent = true;
			form.CurrentCursor = Cursors.SizeAll;

			form.InitializeBoard(AppSetting, SkinSetting, new BoardData("OK : 左ダブクリ\nキャンセル : 右クリ", Skin.SkinRender.NoneArtworkImage));
			form.Preview(1, AppSetting.BoardLocation);

			if(form.ShowDialog() == System.Windows.Forms.DialogResult.OK)
			{
				AppSetting.BoardLocation = form.Location;
				_propertyGrid_AppSetting.SelectedObject = AppSetting;
			}
		}

		/// <summary>
		/// メニューが開くときに呼ばれる
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void skinToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
		{

		}

		/// <summary>
		/// スキンリストのアップデート
		/// </summary>
		private void UpdateSkinList()
		{
			// スキンの読み込み
			_skinManager.SeachSkinFiles(Generic.SkinFolderpath);

			_listBox_Skins.Items.Clear();
			foreach(Skin.SkinInfo info in _skinManager)
			{
				_listBox_Skins.Items.Add(info.SkinSetting.SkinTitle);
			}
		}

		/// <summary>
		/// 選択されているイントロ時間単位によってコントロールの設定値を更新
		/// </summary>
		private void UpdatetIntroCtrlSetting()
		{
			if(SelectedIntroTimeUnit == IntroTimeUnit.Sec)
			{
				_trackBar_IntroStartTime.Maximum = 1000;
				_trackBar_IntroStartTime.Minimum = 0;
				_trackBar_IntroStartTime.TickFrequency = 100;
				_numericUpDown_IntroStartTime.Maximum = 1000;
				_numericUpDown_IntroStartTime.Minimum = 0;

				_trackBar_IntroPlayTime.Maximum = 1000;
				_trackBar_IntroPlayTime.Minimum = 1;
				_trackBar_IntroPlayTime.TickFrequency = 100;
				_numericUpDown_IntroPlayTime.Maximum = 1000;
				_numericUpDown_IntroPlayTime.Minimum = 1;

				_label_IntroStartTime.Text = "開始時間\n[s]";
				_label_IntroPlayTime.Text = "再生時間\n[s]";
			}
			else
			{
				_trackBar_IntroStartTime.Maximum = 100;
				_trackBar_IntroStartTime.Minimum = 0;
				_trackBar_IntroStartTime.TickFrequency = 10;
				_numericUpDown_IntroStartTime.Maximum = 100;
				_numericUpDown_IntroStartTime.Minimum = 0;

				_trackBar_IntroPlayTime.Maximum = 100;
				_trackBar_IntroPlayTime.Minimum = 1;
				_trackBar_IntroPlayTime.TickFrequency = 10;
				_numericUpDown_IntroPlayTime.Maximum = 100;
				_numericUpDown_IntroPlayTime.Minimum = 1;

				_label_IntroStartTime.Text = "開始時間\n[%]";
				_label_IntroPlayTime.Text = "再生時間\n[%]";
			}
		}

		/// <summary>
		/// イントロ時間単位の変更イベント
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void _radioButton_IntroUnitoMode_CheckedChanged(object sender, EventArgs e)
		{
			UpdatetIntroCtrlSetting();
		}

		private void _button_OK_Click(object sender, EventArgs e)
		{

		}
	}
}