﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Microsoft.VisualBasic;
using System.IO;

namespace NauPlaying
{
	/// <summary>
	/// スキンフォルダの入力フォームクラス
	/// </summary>
	public partial class FormInputSkinFoldername : Form
	{
		/// <summary>
		/// スキンマネージャ
		/// </summary>
		private Skin.SkinManager _skinManager;

		/// <summary>
		/// 選択されているスキンを取得。
		/// 選択されていない場合は null
		/// </summary>
		/// <returns></returns>
		private Skin.SkinInfo SelectedSkinSetting
		{
			get
			{
				if(_skinManager.Count == 0)
					return null;

				int idx = _listBox_Skins.SelectedIndex;
				if(idx < 0 || idx >= _skinManager.Count)
					return null;

				return _skinManager[idx];
			}
		}

		/// <summary>
		/// ダイアログに表示するメッセージ
		/// </summary>
		public string Message
		{
			set { _label_Meg.Text = value; }
			get { return _label_Meg.Text; }
		}

		/// <summary>
		/// ダイアログのタイトル
		/// </summary>
		public string Title
		{
			set { Text = value; }
			get { return Text; }
		}

		/// <summary>
		/// スキンフォルダ
		/// </summary>
		public string SkinFoldername { set; get; }
		/// <summary>
		/// スキンフォルダパス
		/// </summary>
		public string SkinFolderpath { set; get; }

		/// <summary>
		/// コンストラクタ
		/// </summary>
		public FormInputSkinFoldername()
		{
			_skinManager = new Skin.SkinManager();
			SkinFoldername = "";

			InitializeComponent();

			UpdateSkinList();
		}

		/// <summary>
		/// OK ボタン
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void _button_OK_Click(object sender, EventArgs e)
		{
			SkinFoldername = _textBox_SkinName.Text;
		}

		/// <summary>
		/// キャンセルボタン
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void _button_Cancel_Click(object sender, EventArgs e)
		{
			SkinFoldername = "";
		}

		/// <summary>
		/// フォームが閉じるときに呼ばれる
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void FormInputSkinName_FormClosing(object sender, FormClosingEventArgs e)
		{
			if(DialogResult == System.Windows.Forms.DialogResult.Cancel)
				return;

			// 文字長チェック
			if(SkinFoldername.Length == 0)
			{
				e.Cancel = true;
				return;
			}

			// 禁則文字のチェック
			if(SkinFoldername.IndexOfAny(Path.GetInvalidFileNameChars()) >= 0)
			{
				MessageBox.Show("スキン名には禁則文字を使用しないでください。", Generic.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
				e.Cancel = true;
				return;
			}

			// フォルダパスの生成
			SkinFolderpath = Path.GetDirectoryName(Application.ExecutablePath) + "\\" + Generic.SkinFoldername + "\\" + SkinFoldername;

			// 上書きの確認
			if(Directory.Exists(SkinFolderpath))
			{
				if(MessageBox.Show("既存のフォルダが存在します。\n上書きしますか？", Generic.AppTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) != System.Windows.Forms.DialogResult.Yes)
					e.Cancel = true;
					return;
			}
		}

		/// <summary>
		/// スキンリストのアップデート
		/// </summary>
		private void UpdateSkinList()
		{
			// スキンの読み込み
			_skinManager.SeachSkinFiles(Generic.SkinFolderpath);

			_listBox_Skins.Items.Clear();
			foreach(Skin.SkinInfo info in _skinManager)
			{
				_listBox_Skins.Items.Add(info.SkinSetting.SkinTitle + "- [Skin\\" + Generic.GetLastFolderName(Path.GetDirectoryName(info.Fullpath)) + "]");
			}
		}

		/// <summary>
		/// スキンリストのダブルクリック
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void _listBox_Skins_DoubleClick(object sender, EventArgs e)
		{
			Skin.SkinInfo selectedInfo = SelectedSkinSetting;
			if(selectedInfo == null)
				return;

			_textBox_SkinName.Text = Generic.GetLastFolderName(Path.GetDirectoryName(selectedInfo.Fullpath));
		}
	}
}
