﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Win32SDK;

namespace NauPlaying
{
	/// <summary>
	/// レイヤードウインドウフォーム基底
	/// </summary>
	public class FormLayeredWindow : Form
	{
		/// <summary>
		/// マウス操作を行うかどうかの設定
		/// </summary>
		public bool IsEnableMouseEvent { set; get; }

		/// <summary>
		/// コンストラクタ
		/// </summary>
		public FormLayeredWindow()
		{
			IsEnableMouseEvent = false;
		}

		/// <summary>
		/// TopMost を設定し ShowWithoutActivation = true としたい場合は Show の変わりにこっちを使用する
		///  ※ Load が発生しないらしいが確認はしていない（使ってないしｗ）
		/// </summary>
		/// <param name="topMost">TopMost 表示したい場合は true</param>
		public void ShowWithoutActiveTopMost(bool topMost)
		{
			if(topMost)
				User32.SetWindowPos(this.Handle, User32.HWND_TOPMOST, 0, 0, 0, 0,
					User32.SWP_NOACTIVATE | User32.SWP_NOMOVE | User32.SWP_NOSIZE | User32.SWP_SHOWWINDOW);
			else
				User32.SetWindowPos(this.Handle, User32.HWND_NULL,
					0, 0, 0, 0, User32.SWP_NOACTIVATE | User32.SWP_NOMOVE | User32.SWP_NOSIZE | User32.SWP_SHOWWINDOW);
		}

		/// <summary>
		/// layerBmp でレイヤードウィンドウを設定します。
		/// </summary>
		/// <param name="layerBmp">表示する画像</param>
		/// <returns>成功した場合は true、失敗した場合は false</returns>
		protected bool SetLayeredWindow(Bitmap layerBmp)
		{
			// デバイスコンテキストを取得
			IntPtr screenDc = IntPtr.Zero;
			IntPtr memDc = IntPtr.Zero;
			IntPtr hBitmap = IntPtr.Zero;
			IntPtr hOldBitmap = IntPtr.Zero;

			try
			{
				screenDc = User32.GetDC(IntPtr.Zero);
				if(screenDc == IntPtr.Zero)
					return false;

				memDc = GDI32.CreateCompatibleDC(screenDc);
				if(memDc == IntPtr.Zero)
					return false;

				hBitmap = layerBmp.GetHbitmap(Color.FromArgb(0));
				hOldBitmap = GDI32.SelectObject(memDc, hBitmap);
				if(hOldBitmap == IntPtr.Zero)
					return false;

				// BLENDFUNCTION を初期化
				User32.BLENDFUNCTION blend = new User32.BLENDFUNCTION();
				blend.BlendOp = User32.AC_SRC_OVER;
				blend.BlendFlags = 0;
				blend.SourceConstantAlpha = 255;
				blend.AlphaFormat = User32.AC_SRC_ALPHA;

				// レイヤードウィンドウを更新
				this.Size = new Size(layerBmp.Width, layerBmp.Height);
				Point pptDst = new Point(this.Left, this.Top);
				Size psize = new Size(this.Width, this.Height);
				Point pptSrc = new Point(0, 0);

				int res = User32.UpdateLayeredWindow(this.Handle, screenDc, ref pptDst, ref psize, memDc, ref pptSrc, 0, ref blend, User32.ULW_ALPHA);
				if(res == 0)
					return false;

				return true;
			}
			finally
			{
				if(screenDc != IntPtr.Zero)
					User32.ReleaseDC(IntPtr.Zero, screenDc);

				if(hBitmap != IntPtr.Zero)
				{
					GDI32.SelectObject(memDc, hOldBitmap);
					GDI32.DeleteObject(hBitmap);
				}

				if(memDc != IntPtr.Zero)
					GDI32.DeleteDC(memDc);
			}
		}

		/// <summary>
		/// ウインドウスタイルをここで書き換えてレイヤードウインドウに設定しています。
		/// </summary>
		protected override System.Windows.Forms.CreateParams CreateParams
		{
			get
			{
				System.Windows.Forms.CreateParams cp = base.CreateParams;

				cp.ExStyle = cp.ExStyle | User32.WS_EX_LAYERED;
				
				if(!IsEnableMouseEvent)
					cp.ExStyle = cp.ExStyle | User32.WS_EX_TRANSPARENT;

				if(this.FormBorderStyle != FormBorderStyle.None)
				{
					cp.Style = cp.Style & (~User32.WS_BORDER);
					cp.Style = cp.Style & (~User32.WS_THICKFRAME);
				}

				return cp;
			}
		}
	}
}
