﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Drawing;

namespace NauPlaying
{
	/// <summary>
	/// float 版簡易 Color 構造体
	/// </summary>
	public struct ColorF
	{
		/// <summary>
		/// アルファ(0 - 255)
		/// </summary>
		public float A;
		/// <summary>
		/// R 要素(0 - 255)
		/// </summary>
		public float R;
		/// <summary>
		/// G 要素(0 - 255)
		/// </summary>
		public float G;
		/// <summary>
		/// B 要素(0 - 255)
		/// </summary>
		public float B;

		/// <summary>
		/// Color 構造体で取得
		/// </summary>
		public Color Color
		{
			get { return ConvColor(this); }
		}

		/// <summary>
		/// コンストラクタ
		/// </summary>
		/// <param name="a">アルファ(0 - 255)</param>
		/// <param name="r">R 要素(0 - 255)</param>
		/// <param name="g">G 要素(0 - 255)</param>
		/// <param name="b">B 要素(0 - 255)</param>
		public ColorF(float a, float r, float g, float b)
		{
			A = a;
			R = r;
			G = g;
			B = b;
		}

		/// <summary>
		/// コンストラクタ
		/// </summary>
		/// <param name="r">R 要素(0 - 255)</param>
		/// <param name="g">G 要素(0 - 255)</param>
		/// <param name="b">B 要素(0 - 255)</param>
		public ColorF(float r, float g, float b)
		{
			A = 0;
			R = r;
			G = g;
			B = b;
		}

		/// <summary>
		/// コンストラクタ
		/// </summary>
		/// <param name="col">生成元 Color 構造体</param>
		public ColorF(Color col)
		{
			A = col.A;
			R = col.R;
			G = col.G;
			B = col.B;
		}

		/// <summary>
		/// Color 構造体に変換する
		/// </summary>
		public static Color ConvColor(ColorF col)
		{
			return ConvColor(col.A, col.R, col.G, col.B);
		}

		/// <summary>
		/// Color 構造体に変換する
		/// </summary>
		public static Color ConvColor(float a, float r, float g, float b)
		{
			int ia = (int)a;
			int ir = (int)r;
			int ig = (int)g;
			int ib = (int)b;

			if(ia > 255) ia = 255;
			if(ia < 0) ia = 0;

			if(ir > 255) ir = 255;
			if(ir < 0) ir = 0;

			if(ig > 255) ig = 255;
			if(ig < 0) ig = 0;

			if(ib > 255) ib = 255;
			if(ib < 0) ib = 0;

			return Color.FromArgb(ia, ir, ig, ib);
		}
	}
}
