﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace NauPlaying
{
	/// <summary>
	/// 共通、汎用処理提供用クラス
	/// </summary>
	public static class Generic
	{
		/// <summary>
		/// アプリケーション名
		/// </summary>
		public const string AppTitle = "なう、ぷれ～いんぐ！";
		/// <summary>
		/// アプリケーション設定ファイル名
		/// </summary>
		public const string AppSettingFilename = "app_setting.xml";
		/// <summary>
		/// iTunes 用アプリケーション設定ファイル名
		/// </summary>
		public const string AppSettingForiTunesFilename = "itunes_setting.xml";
		/// <summary>
		/// Winamp 用アプリケーション設定ファイル名
		/// </summary>
		public const string AppSettingForWinampFilename = "winamp_setting.xml";
		/// <summary>
		/// スキン設定ファイル名
		/// </summary>
		public const string SkinSettingFilename = "skin_setting.xml";
		/// <summary>
		/// アプリケーションが使用するスキン設定ファイルへのフルパスを取得
		/// </summary>
		public static string AppSkinSettingFilepath
		{
			get { return GetFullPath(SkinSettingFilename); }
		}

		/// <summary>
		/// スキンフォルダ名
		/// </summary>
		public const string SkinFoldername = "skin";
		/// <summary>
		/// スキンフォルダへのフルパスを取得
		/// </summary>
		public static string SkinFolderpath
		{
			get { return GetFullPath(SkinFoldername); }
		}

		/// <summary>
		/// プラグインフォルダ名
		/// </summary>
		public const string PluginFoldername = "plugins";
		/// <summary>
		/// プラグインフォルダへのフルパスを取得
		/// </summary>
		public static string PluginFolderpath
		{
			get { return GetFullPath(PluginFoldername); }
		}

		/// <summary>
		/// iTunesMonitorPlugin のアッセンブリ名
		/// </summary>
		public static string iTunesMonitorPluginAssemblyName = "iTunesMonitorPlugin";
		/// <summary>
		/// WinampMonitorPlugin のアッセンブリ名
		/// </summary>
		public static string WinampMonitorPluginAssemblyName = "WinampMonitorPlugin";
		/// <summary>
		/// iTunesMonitorPlugin のクラス名
		/// </summary>
		public static string iTunesMonitorPluginClassName = "PluginBase.iTunesMonitorPlugin";
		/// <summary>
		/// WinampMonitorPlugin のクラス名
		/// </summary>
		public static string WinampMonitorPluginClassName = "PluginBase.WinampMonitorPlugin";
		/// <summary>
		/// SingletonHelper のクラス名
		/// </summary>
		public static string SingletonHelperClassName = "PluginBase.SingletonHelper";

		/*
		/// <summary>
		/// iTunesMonitorPlugin のアッセンブリ名
		/// </summary>
		public static string iTunesMonitorPluginAssemblyName = "iTunesMonitorPlugin";
		/// <summary>
		/// iTunesMonitorPlugin のクラス名
		/// </summary>
		public static string iTunesMonitorPluginClassName = "PluginBase.iTunesMonitorPlugin";
		/// <summary>
		/// WinampMonitorPlugin のアッセンブリ名
		/// </summary>
		public static string WinampMonitorPluginAssemblyName = "WinampMonitorPlugin";
		/// <summary>
		/// WinampMonitorPlugin のクラス名
		/// </summary>
		public static string WinampMonitorPluginClassName = "PluginBase.WinampMonitorPlugin";
		 * */
		/// <summary>
		/// アプリケーションドメインのリモート用のフォルダ
		/// </summary>
		public static string RemoteCachePath = PluginFolderpath + "\\" + "cache" + "\\";
		/// <summary>
		/// ログファイル名
		/// </summary>
		public static string LogFoldrPath = AppPath + Path.DirectorySeparatorChar + "Log.txt";

		/// <summary>
		/// キャッシュの削除
		/// </summary>
		public static void DeleteRemoteCache()
		{
			try
			{
				if(Directory.Exists(Generic.RemoteCachePath))
				{
					Directory.Delete(Generic.RemoteCachePath, true);
				}
			}
			catch(Exception ex)
			{
				UpdateLogFile(ex);
			}
		}

		/// <summary>
		/// ログファイルに例外を追加します
		/// </summary>
		/// <param name="ex">例外</param>
		public static void UpdateLogFile(Exception ex)
		{
			UpdateLogFile(ex.Message, ex.StackTrace);
		}

		/// <summary>
		/// ログファイルを書き込む際のロック
		/// </summary>
		private static object _updateLogFileLockObj = new object();

		/// <summary>
		/// ログファイルにエラーメッセージを追加します。
		/// （スレッドセーフです）
		/// </summary>
		/// <param name="message">エラーメッセージ</param>
		/// <param name="stackTrace">スタックとレースがあれば指定</param>
		public static void UpdateLogFile(string message, string stackTrace)
		{
			lock(_updateLogFileLockObj)
			{
				try
				{
					Encoding sjisEnc = Encoding.GetEncoding("Shift_JIS");
					List<string> lines = new List<string>();

					// 既存のログファイルがある場合はそれを読み込む
					if(File.Exists(Generic.LogFoldrPath))
					{
						using(StreamReader sr = new StreamReader(Generic.LogFoldrPath, sjisEnc))
						{
							string line = "";
							while((line = sr.ReadLine()) != null)
								lines.Add(line);
						}
					}

					// エラー内容を末端に追加
					lines.Add("_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/");
					lines.Add(DateTime.Now.ToString() + " : " + message);
					if(stackTrace.Length > 0)
						lines.Add(stackTrace);

					// 行数制限 (100行に収まるように先頭から削除)
					if(lines.Count > 100)
						lines.RemoveRange(0, lines.Count - 100);

					// ログファイルへ上書き保存
					using(StreamWriter sw = new StreamWriter(Generic.LogFoldrPath, false, sjisEnc))
					{
						foreach(string line in lines)
							sw.WriteLine(line);
					}
				}
				catch(Exception ex)
				{
					MessageBox.Show("ログファイルの操作に失敗しました。\n" + ex.Message, Generic.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				}
			}
		}

		/// <summary>
		/// バックグランドイメージファイル名
		/// </summary>
		public const string BkImageFilename = "BkImage.png";
		/// <summary>
		/// バックグランドイメージファイルへのフルパスを取得
		/// </summary>
		public static string BkImageFilepath
		{
			get { return GetFullPath(BkImageFilename); }
		}

		/// <summary>
		/// アプリケーションのあるフォルダへのフルパスを取得
		/// </summary>
		public static string AppPath
		{
			get { return Path.GetDirectoryName(Application.ExecutablePath); }
		}

		/// <summary>
		/// アプリケーションのあるフォルダ + \filename を生成
		/// </summary>
		/// <param name="filename"></param>
		/// <returns></returns>
		public static string GetFullPath(string filename)
		{
			return AppPath + "\\" + filename;
		}

		/// <summary>
		/// 値→文字列 への変換
		/// </summary>
		/// <typeparam name="T">変換する値の型</typeparam>
		/// <param name="value">変換元の値</param>
		/// <returns></returns>
		public static string ConvertToString<T>(T value)
		{
			return TypeDescriptor.GetConverter(typeof(T)).ConvertToString(value);
		}

		/// <summary>
		/// 文字列→値 への変換
		/// </summary>
		/// <typeparam name="T">変換後の型</typeparam>
		/// <param name="text">変換もとの文字列</param>
		/// <returns></returns>
		public static T ConvertFromString<T>(string text)
		{
			return (T)TypeDescriptor.GetConverter(typeof(T)).ConvertFromString(text);
		}

		/// <summary>
		/// 値が指定の範囲ではない場合修正を行う
		/// </summary>
		/// <param name="value"></param>
		/// <param name="min"></param>
		/// <param name="max"></param>
		/// <returns></returns>
		public static float ModifyinFloatValue(float value, float min, float max)
		{
			if(value < min) return min;
			if(value > max) return max;
			return value;
		}

		/// <summary>
		/// 値が指定の範囲ではない場合修正を行う
		/// </summary>
		/// <param name="value"></param>
		/// <param name="min"></param>
		/// <param name="max"></param>
		/// <returns></returns>
		public static int ModifyinIntValue(int value, int min, int max)
		{
			if(value < min) return min;
			if(value > max) return max;
			return value;
		}

		/// <summary>
		/// 値が指定の範囲ではない場合修正を行う
		/// </summary>
		/// <param name="rect"></param>
		/// <param name="min_x"></param>
		/// <param name="max_x"></param>
		/// <param name="min_y"></param>
		/// <param name="max_y"></param>
		/// <param name="min_width"></param>
		/// <param name="max_width"></param>
		/// <param name="min_height"></param>
		/// <param name="max_height"></param>
		/// <param name="square"></param>
		/// <returns></returns>
		public static Rectangle ModifyingRect(Rectangle rect, int min_x, int max_x, int min_y, int max_y, int min_width, int max_width, int min_height, int max_height, bool square)
		{
			Rectangle result = rect;

			if(result.X < min_x) result.X = min_x;
			if(result.X > max_x) result.X = max_x;
			if(result.Y < min_y) result.Y = min_y;
			if(result.Y > max_y) result.Y = max_y;
			if(result.Width < min_width) result.Width = min_width;
			if(result.Width > max_width) result.Width = max_width;
			if(result.Height < min_height) result.Height = min_height;
			if(result.Height > max_height) result.Height = max_height;

			if(square)
			{
				if(result.Width != result.Height)
				{
					if(result.Width < result.Height)
						result.Height = result.Width;
					else
						result.Width = result.Height;
				}
			}

			return result;
		}

		/// <summary>
		/// 値が指定の範囲ではない場合修正を行う
		/// </summary>
		/// <param name="size"></param>
		/// <param name="min_width"></param>
		/// <param name="max_width"></param>
		/// <param name="min_height"></param>
		/// <param name="max_height"></param>
		/// <returns></returns>
		public static Size ModifyingSize(Size size, int min_width, int max_width, int min_height, int max_height)
		{
			Size result = size;

			if(size.Width < min_width) size.Width = min_width;
			if(size.Width > max_width) size.Width = max_width;
			if(size.Height < min_height) size.Height = min_height;
			if(size.Height > max_height) size.Height = max_height;

			return result;
		}

		/// <summary>
		/// 値が指定の範囲ではない場合修正を行う
		/// </summary>
		/// <param name="pt"></param>
		/// <param name="min_x"></param>
		/// <param name="max_x"></param>
		/// <param name="min_y"></param>
		/// <param name="max_y"></param>
		/// <returns></returns>
		public static Point ModifyingPoint(Point pt, int min_x, int max_x, int min_y, int max_y)
		{
			Point result = pt;

			if(pt.X < min_x) pt.X = min_x;
			if(pt.X > max_x) pt.X = max_x;
			if(pt.Y < min_y) pt.Y = min_y;
			if(pt.Y > max_y) pt.Y = max_y;

			return result;
		}

		/// <summary>
		/// パスから最後のフォルダ名を取り出す
		/// </summary>
		/// <param name="path"></param>
		/// <returns></returns>
		public static string GetLastFolderName(string path)
		{
			string lastFolder = null;

			try
			{
				string[] folders = path.Split('\\');

				if(folders.Length > 0)
					lastFolder = folders[folders.Length - 1];
				else
					lastFolder = "";
			}
			catch(Exception)
			{
				throw;
			}

			return lastFolder;
		}
	}
}
