﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.ComponentModel;

namespace NauPlaying
{
	/// <summary>
	/// スクロールモード
	/// </summary>
	[TypeConverter(typeof(EnumDisplayNameConverter))]
	public enum ScrollMode
	{
		/// <summary>
		/// スクロールしない
		/// </summary>
		[EnumDisplayName("なし (None)")]
		None,
		/// <summary>
		/// 前進スクロール
		/// </summary>
		[EnumDisplayName("前進 (Mode1)")]
		Mode1,
		/// <summary>
		/// 前進→更新スクロール
		/// </summary>
		[EnumDisplayName("往復 (Mode2)")]
		Mode2,
	}

	/// <summary>
	/// 背景イメージモード
	/// </summary>
	[TypeConverter(typeof(EnumDisplayNameConverter))]
	public enum BackImageMode
	{
		/// <summary>
		/// 背景色1で塗りつぶし
		/// </summary>
		[EnumDisplayName("背景色1で塗りつぶし (Solid)")]
		Solid,
		/// <summary>
		/// 上から下へ背景色1→背景色2で塗りつぶし
		/// </summary>
		[EnumDisplayName("上から下へ背景色1→背景色2で塗りつぶし (VerticalGraduation1)")]
		VerticalGraduation1,
		/// <summary>
		/// 上から下へ背景色1→背景色2、アルファを不透明→透明→不透明で変化させて塗りつぶし
		/// </summary>
		[EnumDisplayName("上から下へ背景色1→背景色2、アルファを不透明→透明→不透明で変化させて塗りつぶし (VerticalGraduation2)")]
		VerticalGraduation2,
		/// <summary>
		/// 上から下へ背景色1→背景色2、アルファを透明→不透明→透明で変化させて塗りつぶし
		/// </summary>
		[EnumDisplayName("上から下へ背景色1→背景色2、アルファを透明→不透明→透明で変化させて塗りつぶし (VerticalGraduation3)")]
		VerticalGraduation3,
		/// <summary>
		/// 上から下へ背景色1で、アルファを不透明→透明で変化させて塗りつぶし
		/// </summary>
		[EnumDisplayName("上から下へ背景色1で、アルファを不透明→透明で変化させて塗りつぶし (VerticalGraduation4)")]
		VerticalGraduation4,
		/// <summary>
		/// 上から下へ背景色で1、アルファを透明→不透明で変化させて塗りつぶし
		/// </summary>
		[EnumDisplayName("上から下へ背景色で1、アルファを透明→不透明で変化させて塗りつぶし (VerticalGraduation5)")]
		VerticalGraduation5,
		/// <summary>
		/// 左から右へ背景色1→背景色2で塗りつぶし
		/// </summary>
		[EnumDisplayName("左から右へ背景色1→背景色2で塗りつぶし (HorizontalGraduation1)")]
		HorizontalGraduation1,
		/// <summary>
		/// 左から右へ背景色1→背景色2 、アルファを不透明→透明→不透明で変化させて塗りつぶし
		/// </summary>
		[EnumDisplayName("左から右へ背景色1→背景色2 、アルファを不透明→透明→不透明で変化させて塗りつぶし (HorizontalGraduation2)")]
		HorizontalGraduation2,
		/// <summary>
		/// 左から右へ背景色1→背景色2 、アルファを透明→不透明→透明で変化させて塗りつぶし
		/// </summary>
		[EnumDisplayName("左から右へ背景色1→背景色2 、アルファを透明→不透明→透明で変化させて塗りつぶし (HorizontalGraduation3)")]
		HorizontalGraduation3,
		/// <summary>
		/// 左から右へ背景色1で、アルファを不透明→透明で変化させて塗りつぶし
		/// </summary>
		[EnumDisplayName("左から右へ背景色1で、アルファを不透明→透明で変化させて塗りつぶし (HorizontalGraduation4)")]
		HorizontalGraduation4,
		/// <summary>
		/// 左から右へ背景色1で、アルファを透明→不透明で変化させて塗りつぶし
		/// </summary>
		[EnumDisplayName("左から右へ背景色1で、アルファを透明→不透明で変化させて塗りつぶし (HorizontalGraduation5)")]
		HorizontalGraduation5,
		/// <summary>
		/// 透明
		/// </summary>
		[EnumDisplayName("透明 (Transparence)")]
		Transparence,
	}

	/// <summary>
	/// バックグランドビットマップの透過モード
	/// </summary>
	[TypeConverter(typeof(EnumDisplayNameConverter))]
	public enum BgTransparentMode
	{
		/// <summary>
		/// BgTransparentColor を使用する
		/// </summary>
		[EnumDisplayName("BgTransparentColor を使用する (ConstColor)")]
		ConstColor,
		/// <summary>
		/// 原点(0, 0)をカラーキーとする
		/// </summary>
		[EnumDisplayName("原点(0, 0)をカラーキーとする (OriginColor)")]
		OriginColor,
		/// <summary>
		/// ビットマップの透過設定に従う
		/// </summary>
		[EnumDisplayName("ビットマップの透過設定に従う (Original)")]
		Original,
	}

	/// <summary>
	/// テキストの描画モード
	/// </summary>
	[TypeConverter(typeof(EnumDisplayNameConverter))]
	public enum TextDrawingMode
	{
		/// <summary>
		/// 通常描画
		/// </summary>
		[EnumDisplayName("通常描画 (Normal)")]
		Normal,
		/// <summary>
		/// アウトラインのみ描画
		/// </summary>
		[EnumDisplayName("アウトライン (Outline)")]
		Outline,
		/// <summary>
		/// 縁文字
		/// </summary>
		[EnumDisplayName("縁文字 (Border)")]
		Border,
		/// <summary>
		/// 影つき文字
		/// </summary>
		[EnumDisplayName("影つき (Shadow)")]
		Shadow,
		/// <summary>
		/// 縁文字で影つきの文字
		/// </summary>
		[EnumDisplayName(" 縁文字で影つき (BorderAndShadow)")]
		BorderAndShadow,
	}

	/// <summary>
	/// 使用プレーヤタイプ
	/// </summary>
	[TypeConverter(typeof(EnumDisplayNameConverter))]
	public enum PlayerType
	{
		/// <summary>
		/// 不明、不定なプラグイン
		/// </summary>
		[EnumDisplayName("使用しないでください。(Unknwon)")]
		Unknwon,

		/// <summary>
		/// iTunes を使用
		/// </summary>
		[EnumDisplayName("iTunes")]
		iTunes,
		/// <summary>
		/// Winamp を使用
		/// </summary>
		[EnumDisplayName("Winamp")]
		Winamp,
	}

	/// <summary>
	/// 起動プレーヤチェックの結果（フラグ）
	/// </summary>
	[Flags]
	public enum SerachedPlayerTypeFlags
	{
		/// <summary>
		/// 起動していない
		/// </summary>
		None = 0,
		/// <summary>
		/// iTunesが起動している
		/// </summary>
		iTunes = 1,
		/// <summary>
		/// Winampが起動している
		/// </summary>
		Winamp = 2,

		/// <summary>
		/// すべてのプレーヤが起動している
		/// </summary>
		All = iTunes | Winamp,
	}

	/// <summary>
	/// イントロ再生時間の単位
	/// </summary>
	[TypeConverter(typeof(EnumDisplayNameConverter))]
	public enum IntroTimeUnit
	{
		/// <summary>
		/// 単位を[秒]で指定
		/// </summary>
		[EnumDisplayName("単位を[秒]で指定")]
		Sec,
		/// <summary>
		/// 単位を[%]で指定(総演奏時間に対するパーセンテージ)
		/// </summary>
		[EnumDisplayName("単位を[%]で指定(総演奏時間に対するパーセンテージ)")]
		Percentage,
	}

	/// <summary>
	/// プレーヤーサーチ優先順位
	/// </summary>
	[TypeConverter(typeof(EnumDisplayNameConverter))]
	public enum PlayerSearchPriority
	{
		/// <summary>
		/// 単位を[秒]で指定
		/// </summary>
		[EnumDisplayName("iTunesを優先")]
		iTunes,
		/// <summary>
		/// 単位を[%]で指定(総演奏時間に対するパーセンテージ)
		/// </summary>
		[EnumDisplayName("Winampを優先")]
		Winamp,
	}
}
