﻿using System;
using System.Collections.Generic;
using System.Text;

using System.IO;

namespace NTag.MP3.ID3v2.Frame
{
	/// <summary>
	/// ID3v2 フレームクラス
	/// </summary>
	abstract class Frame
	{
		/// <summary>
		/// テキストデータを持つフレームID
		/// </summary>
		public static readonly Dictionary<NTag.TagFieldFlags, string> TEXTDATA_FRAMEID_TABLE = new Dictionary<NTag.TagFieldFlags, string>
		{
			{ NTag.TagFieldFlags.Title, "TIT2" },
			{ NTag.TagFieldFlags.Artist, "TPE1" },
			{ NTag.TagFieldFlags.Album, "TALB" },
			{ NTag.TagFieldFlags.Year, "TYER" },
			{ NTag.TagFieldFlags.Genre, "TCON" },
			{ NTag.TagFieldFlags.Comment, "COMM" },
			{ NTag.TagFieldFlags.TrackNumber1, "TRCK" },
		};

		/// <summary>
		/// 画像データを持つフレームID
		/// </summary>
		public static readonly Dictionary<NTag.TagFieldFlags, string> PICDATA_FRAMEID_TABLE = new Dictionary<NTag.TagFieldFlags, string>
		{
			{ NTag.TagFieldFlags.Artwork, "APIC" },
		};

		/// <summary>
		/// ヘッダ
		/// </summary>
		public FreamHeader FreamHeader { private set; get; }

		/// <summary>
		/// フレーム内のデータ
		/// </summary>
		public byte[] Data { private set; get; }

		/// <summary>
		/// フレームタイプ
		/// </summary>
		public ID3v2.FrameTpey FrameTpey { private set; get; }

		/// <summary>
		/// コンストラクタ
		/// </summary>
		public Frame(FreamHeader header, byte[] data, ID3v2.FrameTpey frameTpey)
		{
			FreamHeader = header;
			Data = data;
			FrameTpey = frameTpey;
		}

		/// <summary>
		/// フレームの読込み
		/// </summary>
		/// <param name="fs"></param>
		/// <param name="offset"></param>
		/// <param name="frame"></param>
		/// <returns></returns>
		public static int Read(FileStream fs, int offset, out Frame frame)
		{
			try
			{
				frame = null;

				// ヘッダ読込み
				FreamHeader header = new FreamHeader();
				int readBytes = header.Read(fs, offset);

				if(readBytes < 0)
					return -1;

				// データ読込み
				byte[] data = new byte[header.Size];
				fs.Read(data, 0, header.Size);
				readBytes += header.Size;

				// ヘッダにあわせてフレームクラスのタイプをかえる
				if(TEXTDATA_FRAMEID_TABLE.ContainsValue(header.ID))
				{
					frame = new TextFrame(header, data);
				}

				if(frame == null)
				{
					if(PICDATA_FRAMEID_TABLE.ContainsValue(header.ID))
					{
						frame = new PictureFrame(header, data);
					}
				}

				return readBytes;
			}
			catch
			{
				throw;
			}
		}
	}
}
