﻿using System;
using System.Collections.Generic;
using System.Text;

using System.IO;

namespace NTag.MP3.ID3v2.Frame
{
	/// <summary>
	/// フレームヘッダクラス
	/// </summary>
	class FreamHeader
	{
		/// <summary>
		/// ヘッダサイズ（10 固定）
		/// </summary>
		private static readonly int HEADERSIZE = 10;

		/// <summary>
		/// ヘッダのバイトデータ
		/// </summary>
		public byte[] HeaderData { set; get; }

		/// <summary>
		/// フレームID
		/// </summary>
		public string ID { set; get; }

		/// <summary>
		/// フレームのデータサイズ[byte]
		/// （フレームの全サイズはヘッダ 10 バイトを加えたサイズになる。FrmaeSize を使用するといい）
		/// </summary>
		public int Size { set; get; }
		/// <summary>
		/// ヘッダを含めたタグのサイズ[byte]
		/// </summary>
		public int FrmaeSize { get { return Size + HEADERSIZE; } }

		/// <summary>
		/// フラグ
		/// </summary>
		public ID3v2.FreamHeaderFalgs Flags { set; get; }

		/// <summary>
		/// コンストラクタ
		/// </summary>
		public FreamHeader()
		{
			HeaderData = new byte[HEADERSIZE];
			ID = "";
			Size = 0;
			Flags = ID3v2.FreamHeaderFalgs.None;
		}

		/// <summary>
		/// フレームを読み込む
		/// </summary>
		/// <param name="fs">有効なファイルストリーム</param>
		/// <param name="offset">オフセット</param>
		/// <returns>読み込んだバイト数</returns>
		public int Read(FileStream fs, int offset)
		{
			int readBytes = 0;

			if(offset > 0)
				fs.Seek(offset, 0);

			readBytes = fs.Read(HeaderData, 0, HEADERSIZE);
			if(readBytes != HEADERSIZE)
				return -1;

			// ID 取得
			{
				System.Text.Encoding enc = System.Text.Encoding.UTF8;
				ID = enc.GetString(HeaderData, 0, 4);
			}

			// サイズ取得
			{
				int b1 = (int)HeaderData[4] << 24;
				int b2 = (int)HeaderData[5] << 16;
				int b3 = (int)HeaderData[6] << 8;
				int b4 = (int)HeaderData[7];

				Size = b1 | b2 | b3 | b4;
			}

			if(Size <= 0)
				return -1;
			if(Size >= (fs.Length - fs.Position))
				return -1;

			// フラグ取得
			ID3v2.FreamHeaderFalgs flags = (ID3v2.FreamHeaderFalgs)((ushort)(HeaderData[8] << 8) | (ushort)HeaderData[9]);

			return HEADERSIZE;
		}
	}
}
