﻿using System;
using System.Collections.Generic;
using System.Text;

using System.Drawing;

namespace NTag.MP3.ID3v2.Frame
{
	/// <summary>
	/// 画像フレームクラス
	/// </summary>
	class PictureFrame : Frame
	{
		/// <summary>
		/// MIME type
		/// </summary>
		private static readonly string[] MIMETYPE =
		{
			"image/jpg",	// JPEG
			"image/jpeg",	// JPEG
			"image/png",	// PNG
		};

		/// <summary>
		/// MIME type の文字コード
		/// </summary>
		public ID3v2.TextCode TextCode { set; get; }

		/// <summary>
		/// イメージタイプ
		/// </summary>
		public ID3v2.ImageType ImageType { set; get; }

		/// <summary>
		/// 画像タイプ
		/// </summary>
		public ID3v2.PictureType PictureType { set; get; }

		/// <summary>
		/// 画像のデータまでのオフセット
		/// </summary>
		private int _imageDataOffset = 0;
		/// <summary>
		/// 画像データのバイトサイズ
		/// </summary>
		private int _imageDataCount = 0;

		/// <summary>
		/// コンストラクタ
		/// </summary>
		/// <param name="header">フレームヘッダ</param>
		/// <param name="data">フレームデータ</param>
		public PictureFrame(FreamHeader header, byte[] data)
			: base(header, data, ID3v2.FrameTpey.Picture)
		{
			int offset = 0;

			// 文字コードを取得
			TextCode = ID3v2.GetTextCode(Data[0]);
			offset++;

			// MIME Type を取得
			{
				// 0 があるまで読み込む
				int end = offset;
				while(Data[end] != 0)
					end++;

				string mime = ID3v2.GetEncoding(TextCode).GetString(Data, offset, end - offset);

				// JPEG
				if(string.Compare(mime, MIMETYPE[0], true) == 0 || string.Compare(mime, MIMETYPE[1], true) == 0)
				{
					ImageType = ID3v2.ImageType.Jpeg;
				}
				// PNG
				else if(string.Compare(mime, MIMETYPE[2], true) == 0)
				{
					ImageType = ID3v2.ImageType.Png;
				}
				// その他
				else
				{
					ImageType = ID3v2.ImageType.None;
				}

				offset = end + 1;
			}

			// 画像タイプを取得
			PictureType = (ID3v2.PictureType)Data[offset];
			offset+=2;

			// 画像データの取得
			_imageDataOffset = offset;
			_imageDataCount = FreamHeader.Size - offset;
		}

		/// <summary>
		/// イメージの取得
		/// </summary>
		/// <returns></returns>
		public Image GetPicture()
		{
			try
			{
				if(_imageDataCount <= 0)
					return null;

				byte[] tmp = new byte[_imageDataCount];

				Buffer.BlockCopy(Data, _imageDataOffset, tmp, 0, _imageDataCount);

				System.Drawing.ImageConverter conv = new System.Drawing.ImageConverter();
				return (System.Drawing.Image)conv.ConvertFrom(tmp);
			}
			catch
			{
				throw;
			}
		}
	}
}
