﻿using System;
using System.Collections.Generic;
using System.Text;

namespace NTag.MP3.ID3v2.Frame
{
	/// <summary>
	/// テキストデータフレーム
	/// </summary>
	class TextFrame: Frame
	{
		/// <summary>
		/// 文字コード
		/// </summary>
		public ID3v2.TextCode TextCode { set; get; }

		/// <summary>
		/// コンストラクタ
		/// </summary>
		/// <param name="header">フレームヘッダー</param>
		/// <param name="data">フレームデータ</param>
		public TextFrame(FreamHeader header, byte[] data)
			: base(header, data, ID3v2.FrameTpey.Text)
		{
			if(Data.Length > 0)
				TextCode = ID3v2.GetTextCode(Data[0]);
			else
				TextCode = ID3v2.TextCode.None;
		}

		/// <summary>
		/// 現在の文字コードでデコードされた文字列を返す
		/// </summary>
		/// <param name="enc"></param>
		/// <returns></returns>
		public string GetText()
		{
			if(Data.Length <= 0)
				return "";

			int offset = 0;

			// 文字コードを除いてエンコードする
			if(TextCode != ID3v2.TextCode.None)
				offset++;

			int textLength = FreamHeader.Size - offset;
			if(textLength <= 0)
				return "";

			return ID3v2.GetEncoding(TextCode).GetString(Data, offset, textLength);
		}

		/// <summary>
		/// 現在の文字コードでデコードされた文字列を返す
		/// （コメントはこうなっているみたい。他のタグには必要ないと思われる）
		/// </summary>
		/// <param name="enc"></param>
		/// <returns></returns>
		public string GetCommentText()
		{
			if(Data.Length <= 0)
				return "";

			int offset = 0;

			// 国名が入ってる場合それを除去（いまんところ jpn と eng しか見たことがないｗ）
			if(Data.Length >= 4)
			{
				if(ID3v2.GetTextCode(Data[4]) != ID3v2.TextCode.None)
				{
					// 文字コードを除いてエンコードする
					if(TextCode != ID3v2.TextCode.None)
						offset++;

					string code = ID3v2.GetEncoding(TextCode).GetString(Data, offset, 4 - offset);

					if(string.Compare(code, "jpn", true) == 0 || string.Compare(code, "eng", true) == 0)
					{
						offset = 4;
						TextCode = ID3v2.GetTextCode(Data[offset]);
					}
				}
			}

			// 文字コードを除いてエンコードする
			if(TextCode != ID3v2.TextCode.None)
				offset++;

			int textLength = FreamHeader.Size - offset;
			if(textLength <= 0)
				return "";

			return ID3v2.GetEncoding(TextCode).GetString(Data, offset, textLength);
		}
	}
}
