﻿using System;
using System.Collections.Generic;
using System.Text;

using System.IO;
using System.Drawing;

namespace NTag.MP3
{
	public sealed class MP3TagReader : ITagReader
	{
		/// <summary>
		/// タグデータ
		/// </summary>
		public Tag Tag { private set; get; }

		/// <summary>
		/// タグを読み込むファイルのパスの取得
		/// </summary>
		public string FilePath { private set; get; }

		/// <summary>
		/// コンストラクタ
		/// </summary>
		/// <param name="filePath">タグを読み込むファイルのパス</param>
		public MP3TagReader(string filePath)
		{
			FilePath = filePath;
		}

		/// <summary>
		/// 有効な MP3 タグファイルであるかを確認します
		/// </summary>
		/// <returns></returns>
		public bool IsValidFile()
		{
			ITagReader reader = GetTagReader();

			if(reader == null)
				return false;

			return true;
		}

		/// <summary>
		/// タグリーダークラスを取得します。
		/// MP3 タグがない場合は null を返します。
		/// </summary>
		/// <returns></returns>
		private ITagReader GetTagReader()
		{
			ITagReader reader = null;

			// ID3v2 ?
			reader = new ID3v2.ID3v2Reader(FilePath);
			if(!reader.IsValidFile())
			{
				// 1D3v1 ?
				reader = new ID3v1.ID3v1Reader(FilePath);
				if(!reader.IsValidFile())
				{
					// ID3 タグなし
					return null;
				}
			}

			return reader;
		}

		/// <summary>
		/// タグ読み込み
		/// </summary>
		/// <param name="filePath">読み込むファイル名</param>
		public void Read()
		{
			Tag = null;

			try
			{
				ITagReader reader = GetTagReader();

				if(reader == null)
					return;

				reader.Read();
				Tag = reader.Tag;
			}
			catch
			{
				Tag = null;
				throw;
			}
		}
	}
}
