﻿using System;
using System.Collections.Generic;
using System.Text;

using System.Drawing;

namespace NTag.MP4.Atom
{
	/// <summary>
	/// アトムデータクラス
	/// （有効なデータがあるアトム箱のクラスで読める）
	/// </summary>
	sealed class AtomData
	{
		/// <summary>
		/// アトム
		/// </summary>
		public AtomBasic Atom { set; get; }

		/// <summary>
		/// アトムヘッダ
		/// </summary>
		public AtomHeader Header { get { return Atom.Header; } }

		/// <summary>
		/// フラグ
		/// </summary>
		public byte[] Flag { set; get; }

		/// <summary>
		/// 実際のデータサイズ
		/// </summary>
		public int RealDataSize { set; get; }

		/// <summary>
		/// データ（null なので初期化すること）
		/// </summary>
		public byte[] Data { set; get; }

		/// <summary>
		/// コンストラクタ
		/// </summary>
		public AtomData(AtomBasic atom)
		{
			try
			{
				Atom = atom;

				Flag = new byte[4];
				Data = null;

				// 子アトムがある場合はデータなし
				if(Atom.IsHaveChild())
					throw new NTagException(Header.NameStr + " この Atom はデータを持っていません");

				// フラグ
				if(Atom.AtomDataSize >= 4)
				{
					Buffer.BlockCopy(Atom.Data, 0, Flag, 0, 4);
				}

				// 実際のデータが存在する用であればデータをコピー
				if(Atom.AtomDataSize > 8)
				{
					RealDataSize = Atom.AtomDataSize - 8;

					Data = new byte[RealDataSize];
					Buffer.BlockCopy(Atom.Data, 8, Data, 0, RealDataSize);
				}
			}
			catch
			{
				throw;
			}
		}

		/// <summary>
		/// アトムデータを文字列に変換する
		/// </summary>
		/// <returns></returns>
		public string GetText()
		{
			return Encoding.UTF8.GetString(Data);
		}

		/// <summary>
		/// アトムデータを画像に変換する
		/// </summary>
		/// <returns></returns>
		public Image GetPicture()
		{
			System.Drawing.ImageConverter conv = new System.Drawing.ImageConverter();
			return (System.Drawing.Image)conv.ConvertFrom(Data);
		}
	}
}
