﻿using System;
using System.Collections.Generic;
using System.Text;

namespace NTag.MP4.Atom
{
	/// <summary>
	/// アトムヘッダクラス
	/// </summary>
	sealed class AtomHeader
	{
		/// <summary>
		/// アトムサイズ
		/// </summary>
		public int Size { set; get; }
		/// <summary>
		/// アトム名
		/// </summary>
		public byte[] Name { set; get; }

		/// <summary>
		/// アトム名（文字列版）
		/// </summary>
		public string NameStr;

		/// <summary>
		/// コンストラクタ
		/// </summary>
		public AtomHeader()
		{
			Name = new byte[4];
			NameStr = "";
		}

		/// <summary>
		/// ヘッダを読み込む
		/// </summary>
		/// <param name="data"></param>
		/// <param name="offset"></param>
		/// <returns></returns>
		public int ReadHeader(byte[] data, int offset)
		{
			Encoding enc = System.Text.Encoding.ASCII;
			byte[] tmp = new byte[4];

			// Atom サイズ offset +0byte
			Buffer.BlockCopy(data, offset, tmp, 0, 4);
			Array.Reverse(tmp);
			Size = BitConverter.ToInt32(tmp, 0);

			// Atom 名 offset +4byte
			Buffer.BlockCopy(data, offset + 4, Name, 0, 4);
			NameStr = enc.GetString(Name);

			return 8;
		}

		/// <summary>
		/// アトム名が 0x00 * 4 の NULL スペースであるかを調べる
		/// </summary>
		/// <returns></returns>
		public bool IsNullAtomName()
		{
			return Function.ByteArrayCompare(Name, Atom.NULL_ATOMNAME);
		}
	}
}
