﻿using System;
using System.Collections.Generic;
using System.Text;

using System.IO;

namespace NTag.MP4
{
	/// <summary>
	/// MPEG4 ボックスデータクラス
	/// </summary>
	sealed class BoxData
	{
		// ボックスサイズ
		public int Size { set; get; }
		// ボックスタイプ（4バイトで初期化済み）
		public byte[] Type { set; get; }
		// ボックスタイプ（文字列）
		public string TypeStr;

		// ボックスデータ（null なので初期化すること）
		public byte[] Data { set; get; }

		/// <summary>
		/// コンストラクタ
		/// </summary>
		public BoxData()
		{
			Type = new byte[4];
			TypeStr = "";
			Data = null;
		}

		/// <summary>
		/// ボックスデータを読み込む
		/// </summary>
		/// <param name="fs">ファイルストリーム</param>
		/// <param name="offset">読み込み位置</param>
		/// <returns></returns>
		public int Read(FileStream fs, int offset)
		{
			Encoding enc = System.Text.Encoding.ASCII;
			int readBytes = 0;
			
			if(offset > 0)
				fs.Seek(offset, 0);

			// ボックスサイズ
			{
				byte[] tmp = new byte[4];
				readBytes += fs.Read(tmp, 0, 4);

				Array.Reverse(tmp);
				Size = BitConverter.ToInt32(tmp, 0);
			}

			// ボックスタイプ
			{
				readBytes += fs.Read(Type, 0, 4);
				TypeStr = enc.GetString(Type);
			}

			// ボックスデータ
			{
				int dataSize = Size - 8;

				// 読み込み
				if(dataSize < 0)
					return -1;
				if(dataSize > (fs.Length - fs.Position))
					return -1;

				Data = new byte[dataSize];
				readBytes += fs.Read(Data, 0, dataSize);
			}

			return readBytes;
		}
	}
}
