﻿using System;
using System.Collections.Generic;
using System.Text;


namespace NTag
{
	/// <summary>
	/// 汎用の定義
	/// </summary>
	public sealed class NTag
	{
		[Flags]
		public enum TagFieldFlags : int
		{
			/// <summary>
			/// なし（初期値）
			/// </summary>
			None = 0,

			/// <summary>
			/// タイトル
			/// </summary>
			Title = 0x00000001,
			/// <summary>
			/// アーティスト
			/// </summary>
			Artist = 0x00000002,
			/// <summary>
			/// アルバム
			/// </summary>
			Album = 0x00000004,
			/// <summary>
			/// ジャンル
			/// </summary>
			Genre = 0x00000008,
			/// <summary>
			/// 年
			/// </summary>
			Year = 0x00000010,
			/// <summary>
			/// コメント
			/// </summary>
			Comment = 0x00000020,
			/// <summary>
			/// トラック番号
			/// </summary>
			TrackNumber1 = 0x00000040,
			/// <summary>
			/// トラック番号（全トラック数）
			/// </summary>
			TrackNumber2 = 0x00000080,
			/// <summary>
			/// トラック番号文字列（タグによっては文字列で格納されている場合があるため）
			/// </summary>
			TrackNumberString = 0x00000100,

			/// <summary>
			/// アートワーク
			/// </summary>
			Artwork = 0x00000200,

		}

		/// <summary>
		/// タグの種類
		/// </summary>
		[Flags]
		public enum TagType : int
		{
			/// <summary>
			/// なし（初期値）
			/// </summary>
			None = 0,

			/// <summary>
			/// ID3 ver 1.0 または 1.1
			/// </summary>
			ID3V1	= 0x00000001,
			/// <summary>
			/// ID3 ver 2.3
			/// </summary>
			ID3V2_3 = 0x00000100 | ID3V2_x,
			/// <summary>
			/// ID3 ver 2.4
			/// </summary>
			ID3V2_4 = 0x00000200 | ID3V2_x,
			/// <summary>
			/// ID3 ver 2.x
			/// </summary>
			ID3V2_x	= 0x00008000,

			/// <summary>
			/// AAC タグ
			/// </summary>
			AAC		= 0x00800000,
		}

		/// <summary>
		/// タグリーダーを選択する方法
		/// </summary>
		public enum ReadSelectType
		{
			/// <summary>
			/// ファイル構造で選択
			/// </summary>
			FileStruct,
			/// <summary>
			/// ファイル拡張子で選択
			/// </summary>
			Extension,
		}
	}
}
