﻿using System;
using System.Collections.Generic;
using System.Text;

using System.Drawing;

namespace NTag
{
	/// <summary>
	/// タグデータクラス
	/// </summary>
	public sealed class Tag
	{
		

		/// <summary>
		/// タグ読み込み元のファイルパス
		/// </summary>
		public string FilePath { set; get; }

		/// <summary>
		/// タグのタイプ
		/// </summary>
		public NTag.TagType TagType { set; get; }

		/// <summary>
		/// タイトル
		/// </summary>
		public string Title { set; get; }
		/// <summary>
		/// アーティスト
		/// </summary>
		public string Artist { set; get; }
		/// <summary>
		/// アルバム
		/// </summary>
		public string Album { set; get; }
		/// <summary>
		/// ジャンル
		/// </summary>
		public string Genre { set; get; }
		/// <summary>
		/// 年
		/// </summary>
		public string Year { set; get; }
		/// <summary>
		/// コメント
		/// </summary>
		public string Comment { set; get; }
		/// <summary>
		/// トラック番号
		/// </summary>
		public int TrackNumber1 { set; get; }
		/// <summary>
		/// トラック番号（全トラック数）
		/// </summary>
		public int TrackNumber2 { set; get; }
		/// <summary>
		/// トラック番号文字列（タグによっては文字列で格納されている場合があるため）
		/// </summary>
		public string TrackNumberString { set; get; }

		/// <summary>
		/// アートワーク
		/// </summary>
		public Image Artwork { set; get; }
		/// <summary>
		/// アートワークリスト
		/// （複数ある場合にはここに格納される）
		/// </summary>
		public List<Image> Artworks { private set; get; }

		/// <summary>
		/// コンストラクタ
		/// </summary>
		public Tag()
		{
			FilePath = "";

			TagType = NTag.TagType.None;

			Title = "";
			Artist = "";
			Album = "";
			Genre = "";
			Year = "";
			Comment = "";
			TrackNumber1 = 0;
			TrackNumber2 = 0;
			TrackNumberString = "";

			Artwork = null;
			Artworks = new List<Image>();
		}

		/// <summary>
		/// コピーコンストラクタ
		/// </summary>
		/// <param name="src"></param>
		public Tag(Tag src)
			: this()
		{
			CopyFrom(src);
		}

		/// <summary>
		/// タグ情報をコピーする
		/// </summary>
		/// <param name="src">コピー元のタグ情報</param>
		public void CopyFrom(Tag src)
		{
			FilePath = src.FilePath;

			TagType = src.TagType;

			Title = src.Title;
			Artist = src.Artist;
			Album = src.Album;
			Genre = src.Genre;
			Year = src.Year;
			Comment = src.Comment;
			TrackNumber1 = src.TrackNumber1;
			TrackNumber2 = src.TrackNumber2;
			TrackNumberString = src.TrackNumberString;


			if(src.Artwork != null)
				Artwork = new Bitmap(src.Artwork);
			else
			{
				if(Artwork != null)
				{
					Artwork.Dispose();
					Artwork = null;
				}
			}

			Artworks.Clear();
			foreach(Image image in Artworks)
				Artworks.Add(new Bitmap(image));
		}
	}
}
