﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Collections.Specialized;
using System.IO;

namespace NauPlaying.Skin
{
	

	/// <summary>
	/// スキンマネージャクラス
	/// </summary>
	public class SkinManager
	{
		/// <summary>
		/// スキンリスト
		/// </summary>
		List<SkinInfo> _skins = null;

		/// <summary>
		/// 検索されたスキン個数
		/// </summary>
		public int Count
		{ get { return _skins.Count; } }

		/// <summary>
		/// スキン情報を取得するためのインデクサ
		/// </summary>
		/// <param name="i"></param>
		/// <returns></returns>
		public SkinInfo this[int i]
		{
			get { return _skins[i]; }
		}

		/// <summary>
		/// 標準のコンストラクタ
		/// </summary>
		public SkinManager()
		{
			_skins = new List<SkinInfo>();
		}

		/// <summary>
		/// 反復処理のための列挙子を返す
		/// </summary>
		/// <returns></returns>
		public IEnumerator<SkinInfo> GetEnumerator()
		{
			return _skins.GetEnumerator();
		}

		/// <summary>
		/// 検索ルートディレクトリ内、サブフォルダも含めてファイル検索を行う
		/// </summary>
		/// <param name="rootPath">検索ルートディレクトリ</param>
		/// <param name="pattern">検索パターン</param>
		/// <returns></returns>
		private static string[] SearchFiles(string rootPath, string pattern)
		{
			StringCollection result = new StringCollection();

			// 初回ファイル検索
			string[] root_files = Directory.GetFiles(rootPath, pattern);
			foreach(string file in root_files)
			{
				result.Add(file);
			}

			// サブディレクトリ検索
			string[] dirs = Directory.GetDirectories(rootPath);

			foreach(string dir in dirs)
			{
				// 再起呼び出し
				string[] files = SearchFiles(dir, pattern);

				// 条件に合致したファイルがあった場合は、ArrayList に加える
				if(files != null)
				{
					result.AddRange(files);
				}
			}

			string[] find_files = new string[result.Count];
			result.CopyTo(find_files, 0);
			return find_files;
		}

		/// <summary>
		/// スキンを検索します
		/// </summary>
		/// <param name="skinFolder">スキンフォルダ</param>
		public void SeachSkinFiles(string skinFolder)
		{
			_skins.Clear();

			string[] filepaths = null;
			try
			{
				filepaths = SearchFiles(skinFolder, "skin_setting.xml");
				if(filepaths == null)
					return;
			}
			catch(Exception)
			{
				return;
			}

			foreach(string filepath in filepaths)
			{
				try
				{
					SkinSetting skin = SkinSetting.Deserialize(filepath);
					_skins.Add(new SkinInfo(filepath, skin));
				}
				catch(Exception)
				{
				}
			}
		}
	}
}
