﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Drawing;
using System.ComponentModel;
using System.Xml.Serialization;
using System.IO;

namespace NauPlaying.Skin
{
	/// <summary>
	/// スキン設定
	/// </summary>
	[TypeConverter(typeof(PropertyDisplayConverter))]
	public class SkinSetting
	{
		/// <summary>
		/// スキンの著作権表示
		/// </summary>
		[PropertyDisplayName("01 : 著作権")]
		[Description("著作権"), Category("01 : このスキンに関して")]
		public string Copyright { set; get; }
		/// <summary>
		/// 作者コメント
		/// </summary>
		[PropertyDisplayName("02 : コメント")]
		[Description("コメント"), Category("01 : このスキンに関して")]
		public string Comment { set; get; }
		/// <summary>
		/// スキンタイトル
		/// </summary>
		[PropertyDisplayName("03 : タイトル")]
		[Description("タイトル"), Category("01 : このスキンに関して")]
		public string SkinTitle { set; get; }
		/// <summary>
		/// スキンバージョン
		/// </summary>
		[PropertyDisplayName("04 : バージョン")]
		[Description("バージョン"), Category("01 : このスキンに関して")]
		public string Version { set; get; }

		/// <summary>
		/// テキストフォント
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("01 : テキストフォント")]
		[Description("テキストフォント"), Category("02 : テキスト設定")]
		public Font TextFont { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string TextFontAsString
		{
			get { return Generic.ConvertToString(TextFont); }
			set { TextFont = Generic.ConvertFromString<Font>(value); }
		}
		/// <summary>
		/// テキスト色1
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("02 : テキスト色1")]
		[Description("テキスト色"), Category("02 : テキスト設定")]
		public Color TextColor1 { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string TextColor1AsString
		{
			get { return Generic.ConvertToString(TextColor1); }
			set { TextColor1 = Generic.ConvertFromString<Color>(value); }
		}
		/// <summary>
		/// テキスト色2
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("03 : テキスト色2")]
		[Description("テキスト色2"), Category("02 : テキスト設定")]
		public Color TextColor2 { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string TextColor2AsString
		{
			get { return Generic.ConvertToString(TextColor2); }
			set { TextColor2 = Generic.ConvertFromString<Color>(value); }
		}
		/// <summary>
		/// テキストの描画モード
		/// </summary>
		[PropertyDisplayName("04 : テキスト描画モード")]
		[Description(
			"テキストの描画モード\n" +
			"Normal : テキスト色1で普通に描画。\n" +
			"Outline : アウトライン（縁）のみをテキスト色2で描画。\n" +
			"Border : フチ文字描画。文字部分をテキスト色1、縁をテキスト色2で描画。\n" +
			"Shadow : 影つき文字描画。文字部分をテキスト色1、影をテキスト色2で描画。"), Category("02 : テキスト設定")]
		public TextDrawingMode TextDrawingMode { set; get; }
		/// <summary>
		/// テキストアウトライン表示幅
		/// </summary>
		[PropertyDisplayName("05 : アウトライン表示幅")]
		[Description(
			"テキストのアウトライン線幅 1 ～ 10で設定\n" +
			"TextDrawingMode でどのモードを選択しているかで意味が異なります。\n" +
			"Normal : 使用されません。\n" +
			"Outline : アウトライン（縁）の描画幅となります。\n" +
			"Border : アウトライン（縁）の描画幅となります。\n" +
			"Shadow : 影のずれとして使用されます。"), Category("02 : テキスト設定")]
		public int OutlineWidth { set; get; }

		/// <summary>
		/// 背景イメージモード
		/// </summary>
		[PropertyDisplayName("01 : 背景イメージモード")]
		[Description(
			"背景モード\n" +
			"Solid : 背景色1で塗りつぶし。\n" +
			"VerticalGraduation1 : 上から下へ背景色1→背景色2でグラデーション。\n" +
			"VerticalGraduation2 : 上から下へ背景色1でアルファのみを不透明→透明でグラデーション。\n" +
			"VerticalGraduation3 : 上から下へ背景色1でアルファのみを透明→不透明でグラデーション。\n" +
			"VerticalGraduation1～3 : 左から右へのグラデーション。1～3の効果はVerticalGraduationと同じです。\n" +
			"Transparence : 透明。"), Category("03 : 背景設定")]
		public BackImageMode BackImageMode { set; get; }
		/// <summary>
		/// 背景色1
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("02 : 背景色1")]
		[Description("背景色1\n通常の背景色。"), Category("03 : 背景設定")]
		public Color BackColor1 { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string BackColor1AsString
		{
			get { return Generic.ConvertToString(BackColor1); }
			set { BackColor1 = Generic.ConvertFromString<Color>(value); }
		}
		/// <summary>
		/// 背景色2
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("03 : 背景色2")]
		[Description("背景色2\nVerticalGraduation1 または HorizontalGraduation1 の時に使用されるがそれ以外は使用されない。"), Category("03 : 背景設定")]
		public Color BackColor2 { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string BackColor2AsString
		{
			get { return Generic.ConvertToString(BackColor2); }
			set { BackColor2 = Generic.ConvertFromString<Color>(value); }
		}
		/// <summary>
		/// グラデーション増加係数
		/// </summary>
		[PropertyDisplayName("04 : グラデーション増加係数")]
		[Description("グラデーション効果の係り具合\n0 ～ 10で設定。1 以下で変化が小さく、1 以上で変化が大きくなる。"), Category("03 : 背景設定")]
		public float GraduationFactor { set; get; }
		/// <summary>
		/// バックグランドイメージを表示するか
		/// </summary>
		[PropertyDisplayName("05 : バックグランドイメージの表示")]
		[Description("バックグランドイメージを表示するか\ntrue に設定した場合は BgImage.png を必ず作ること。"), Category("03 : 背景設定")]
		public bool EnableBgImage { set; get; }
		/// <summary>
		/// バックグランドイメージの透過色
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("06 : バックグランド透過色")]
		[Description("バックグランドイメージの透過色\nBgTransparentMode で ConstColor を設定した場合に有効。"), Category("03 : 背景設定")]
		public Color BgTransparentColor { set; get; }

		[EditorBrowsable(EditorBrowsableState.Never)]
		[Browsable(false)]
		public string BgTransparentColorAsString
		{
			get { return Generic.ConvertToString(BgTransparentColor); }
			set { BgTransparentColor = Generic.ConvertFromString<Color>(value); }
		}
		/// <summary>
		/// バッググランドイメージの透過設定
		/// </summary>
		[PropertyDisplayName("07 : バッググランド透過設定")]
		[Description(
			"バックグランドイメージの透過設定\n" +
			"ConstColor : BgTransparentColor を透過色とする。\n" +
			"OriginColor : イメージの原点のピクセル色を透過色とする。\n" +
			"Original : イメージをそのまま描画。透過イメージはそのまま透過されます。"), Category("03 : 背景設定")]
		public BgTransparentMode BgTransparentMode { set; get; }
		/// <summary>
		/// バックグランドイメージを背景とするか前景とするか （未使用）
		/// </summary>
		[XmlIgnore]
		[PropertyDisplayName("08 : バックグランドイメージを背景とするか前景とするか")]
		[Browsable(false)]
		[Description("バックグランドイメージを背景とするか前景とするか\ntrue : 背景\nfalse : 前景"), Category("03 : 背景設定")]
		public bool BgDrawingBack { set; get; }
		
		/// <summary>
		/// ボードサイズ
		/// </summary>
		[PropertyDisplayName("01 : ボードサイズ")]
		[Description("ボードサイズ"), Category("04 : ボード設定")]
		public Size BoardSize { set; get; }
		/// <summary>
		/// テキスト表示領域
		/// </summary>
		[PropertyDisplayName("02 : テキスト表示領域")]
		[Description("テキスト表示領域"), Category("04 : ボード設定")]
		public Rectangle TextDispRect { set; get; }
		/// <summary>
		/// アートワーク表示領域
		/// </summary>
		[PropertyDisplayName("03 : アートワーク表示領域")]
		[Description("アートワーク表示領域\n正方で指定してください。"), Category("04 : ボード設定")]
		public Rectangle ArtworkDispRect { set; get; }
		/// <summary>
		/// バッググランドイメージの表示領域
		/// </summary>
		[PropertyDisplayName("04 : バッググランド表示領域")]
		[Description("バックグランドイメージの表示領域"), Category("04 : ボード設定")]
		public Rectangle BgImageDispRect { set; get; }
		/// <summary>
		/// アートワークを表示するかの設定
		/// </summary>
		[PropertyDisplayName("05 : アートワークの表示設定")]
		[Description("アートワークを表示するか\ntrue でアートワークがあった場合に表示する。"), Category("04 : ボード設定")]
		public bool EnableArtwork { set; get; }
		

		/// <summary>
		/// 標準のコンストラクタ
		/// </summary>
		public SkinSetting()
		{
			Copyright = "none";
			Comment = "スキンの雛形です";
			SkinTitle = "template skin";
			Version = "0";

			BoardSize = new Size(512, 96);

			EnableArtwork = true;

			BackImageMode = NauPlaying.BackImageMode.VerticalGraduation4;
			BackColor1 = Color.FromArgb(0, 32, 192);
			BackColor2 = Color.FromArgb(0, 0, 0);
			GraduationFactor = 1.5f;

			EnableBgImage = false;
			BgTransparentColor = Color.Black;
			BgTransparentMode = NauPlaying.BgTransparentMode.Original;
			BgImageDispRect = new Rectangle(BoardSize.Width - BoardSize.Height, 0, BoardSize.Height, BoardSize.Height);
			BgDrawingBack = true;

			int margin = 3;

			ArtworkDispRect = new Rectangle(margin, margin, BoardSize.Height - margin * 2, BoardSize.Height - margin * 2);

			int x_pos = ArtworkDispRect.Right + margin;
			TextDispRect = new Rectangle(x_pos, margin, BoardSize.Width - x_pos - margin, BoardSize.Height - margin * 2);

			TextFont = new Font("ＭＳ ゴシック", 25);
			TextColor1 = Color.FromArgb(255, 255, 64);
			TextColor2 = Color.FromArgb(64, 64, 0);
			TextDrawingMode = NauPlaying.TextDrawingMode.Border;
			OutlineWidth = 2;
		}

		/// <summary>
		/// シリアライズ
		/// </summary>
		/// <param name="filename">xlmファイル名</param>
		public void Serialize(string filename)
		{
			System.Xml.Serialization.XmlSerializer serializer = new System.Xml.Serialization.XmlSerializer(typeof(SkinSetting));

			using(System.IO.FileStream fs = new System.IO.FileStream(filename, System.IO.FileMode.Create))
			{
				serializer.Serialize(fs, this);
			}
		}

		/// <summary>
		/// デシリアライズ
		/// </summary>
		/// <param name="filename">xlmファイル名</param>
		/// <returns></returns>
		public static SkinSetting Deserialize(string filename)
		{
			System.Xml.Serialization.XmlSerializer serializer = new System.Xml.Serialization.XmlSerializer(typeof(SkinSetting));

			using(System.IO.FileStream fs = new System.IO.FileStream(filename, System.IO.FileMode.Open))
			{
				SkinSetting setting = (SkinSetting)serializer.Deserialize(fs);
				return setting;
			}
		}

		/// <summary>
		/// ディープコピー
		/// </summary>
		/// <returns></returns>
		public SkinSetting DeepCopy()
		{
			SkinSetting obj = (SkinSetting)MemberwiseClone();
			
			return obj;
		}
	}
}
