﻿using System;
using System.Drawing;
using System.Text;

using System.Runtime.InteropServices;

namespace Win32SDK
{
	internal static class GDI32
	{
		public const int SRCCOPY = 13369376;

		/// <summary>
		/// Win32SDK CreateCompatibleDC : 指定されたデバイスと互換性のあるメモリデバイスコンテキストを作成します。
		/// </summary>
		/// <param name="hdc">デバイスコンテキストのハンドル</param>
		/// <returns>関数が成功すると、作成されたメモリデバイスコンテキストのハンドルが返ります。関数が失敗すると、NULL が返ります。</returns>
		public static IntPtr CreateCompatibleDC(IntPtr hdc)
		{ return NativeMethods.CreateCompatibleDC(hdc); }

		/// <summary>
		/// Win32SDK SelectObject : 指定されたデバイスコンテキストで、指定された 1 個のオブジェクトを選択します。
		/// 新しいオブジェクトは、同じタイプの以前のオブジェクトを置き換えます。
		/// </summary>
		/// <param name="hdc">デバイスコンテキストのハンドル</param>
		/// <param name="hgdiobj">オブジェクトのハンドル</param>
		/// <returns>リージョン以外のオブジェクトを指定した場合に関数が成功すると、置き換えが発生する前のオブジェクトのハンドルが返ります。</returns>
		public static IntPtr SelectObject(IntPtr hdc, IntPtr hgdiobj)
		{ return NativeMethods.SelectObject(hdc, hgdiobj); }

		/// <summary>
		/// Win32SDK DeleteObject : ペン、ブラシ、フォント、ビットマップ、リージョン、パレットの
		/// いずれかの論理オブジェクトを削除し、そのオブジェクトに関連付けられていたすべてのシステムリソースを解放します。
		/// オブジェクトを削除した後は、指定されたハンドルは無効になります。
		/// </summary>
		/// <param name="hobject">グラフィックオブジェクトのハンドル</param>
		/// <returns>関数が成功すると、0 以外の値が返ります。</returns>
		public static int DeleteObject(IntPtr hobject)
		{ return NativeMethods.DeleteObject(hobject); }

		/// <summary>
		/// Win32SDK DeleteDC : 指定されたデバイスコンテキストを削除します。
		/// </summary>
		/// <param name="hdc">デバイスコンテキストのハンドル</param>
		/// <returns>関数が成功すると、0 以外の値が返ります。</returns>
		public static int DeleteDC(IntPtr hdc)
		{ return NativeMethods.DeleteDC(hdc); }

		/// <summary>
		/// Win32SDK BitBlt : ビットブロック転送を行います。
		/// コピー元からコピー先のデバイスコンテキストへ、指定された長方形内の
		/// 各ピクセルの色データをコピーします。
		/// </summary>
		/// <param name="hDestDC">コピー先デバイスコンテキストのハンドル</param>
		/// <param name="x">コピー先長方形の左上隅の x 座標</param>
		/// <param name="y">コピー先長方形の左上隅の y 座標</param>
		/// <param name="nWidth">コピー先長方形の幅</param>
		/// <param name="nHeight">コピー先長方形の高さ</param>
		/// <param name="hSrcDC">コピー元デバイスコンテキストのハンドル</param>
		/// <param name="xSrc">コピー元長方形の左上隅の x 座標</param>
		/// <param name="ySrc">コピー元長方形の左上隅の y 座標</param>
		/// <param name="dwRop">ラスタオペレーションコード</param>
		/// <returns>関数が成功すると、0 以外の値が返ります。</returns>
		public static int BitBlt(IntPtr hDestDC, int x, int y, int nWidth, int nHeight, IntPtr hSrcDC, int xSrc, int ySrc, int dwRop)
		{ return NativeMethods.BitBlt(hDestDC, x, y, nWidth, nHeight, hSrcDC, xSrc, ySrc, dwRop); }

		/// <summary>
		/// NativeMethods ネイティブコードを呼び出す内部クラス
		/// </summary>
		internal static class NativeMethods
		{
			[DllImport("gdi32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern IntPtr CreateCompatibleDC(IntPtr hdc);

			[DllImport("gdi32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern IntPtr SelectObject(IntPtr hdc, IntPtr hgdiobj);

			[DllImport("gdi32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern int DeleteObject(IntPtr hobject);

			[DllImport("gdi32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern int DeleteDC(IntPtr hdc);

			[DllImport("gdi32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern int BitBlt(IntPtr hDestDC, int x, int y, int nWidth, int nHeight, IntPtr hSrcDC, int xSrc, int ySrc, int dwRop);

		}
	}
}
