﻿using System;
using System.Drawing;
using System.Text;

using System.Runtime.InteropServices;

namespace Win32SDK
{
	internal static class Kernel32
	{
		public const UInt32 FORMAT_MESSAGE_FROM_SYSTEM = 0x00001000;

		/// <summary>
		/// メッセージ文字列を書式化します（書式を割り当てます）。この関数は、入力としてメッセージ定義を受け取ります。
		/// </summary>
		/// <param name="dwFlags">入力元と処理方法のオプション</param>
		/// <param name="lpSource">メッセージの入力元</param>
		/// <param name="dwMessageId">メッセージ識別子</param>
		/// <param name="dwLanguageId">言語識別子</param>
		/// <param name="lpBuffer">メッセージバッファ</param>
		/// <param name="nSize">メッセージバッファの最大サイズ</param>
		/// <param name="Arguments">複数のメッセージ挿入シーケンスからなる配列</param>
		/// <returns>関数が成功すると、バッファに格納された TCHAR 単位の文字数が返ります。終端の NULL 文字は、この数に含まれません。関数が失敗すると、0 が返ります。</returns>
		public static UInt32 FormatMessage(UInt32 dwFlags, IntPtr lpSource, UInt32 dwMessageId, UInt32 dwLanguageId, StringBuilder lpBuffer, int nSize, IntPtr Arguments)
		{ return FormatMessage(dwFlags, lpSource, dwMessageId, dwLanguageId, lpBuffer, nSize, Arguments); }

		/// <summary>
		/// Win32SDK のエラーコードの説明文を取得
		/// </summary>
		/// <returns></returns>
		public static string FormatMessageFromLastError()
		{
			int errCode = Marshal.GetLastWin32Error();
			StringBuilder msg = new StringBuilder(255);
			FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, IntPtr.Zero, (uint)errCode, 0, msg, msg.Capacity, IntPtr.Zero);

			return errCode.ToString() + " : " + msg.ToString();
		}

		/// <summary>
		/// 呼び出し側プロセスのアドレス空間に該当ファイルがマップされている場合、指定されたモジュール名のモジュールハンドルを返します。
		/// </summary>
		/// <param name="lpModuleName">［入力］モジュール（.DLL または .EXE ファイル）の名前を保持する、NULL で終わる文字列へのポインタを指定します。</param>
		/// <returns></returns>
		public static IntPtr GetModuleHandle(string lpModuleName)
		{ return GetModuleHandle(lpModuleName); }

		/// <summary>
		/// 指定されたプロセスのメモリ領域からデータを読み取ります。読み取られる領域全体がアクセス可能でなければなりません。さもないと、関数は失敗します。
		/// </summary>
		/// <param name="hProcess">プロセスのハンドル</param>
		/// <param name="lpBaseAddress">読み取り開始アドレス</param>
		/// <param name="lpBuffer">データを格納するバッファ</param>
		/// <param name="dwSize">読み取りたいバイト数</param>
		/// <param name="lpNumberOfBytesRead">読み取ったバイト数</param>
		/// <returns>関数が成功すると、0 以外の値が返ります。</returns>
		public static bool ReadProcessMemory(IntPtr hProcess, IntPtr lpBaseAddress, [Out] byte[] lpBuffer, UInt32 dwSize, out UInt32 lpNumberOfBytesRead)
		{ return ReadProcessMemory(hProcess, lpBaseAddress, lpBuffer, dwSize, out lpNumberOfBytesRead); }


		[Flags]
		public enum ProcessAccessFlags : uint
		{
			All = 0x001f0fff,
			Terminate = 0x00000001,
			CreateThread = 0x00000002,
			VMOperation = 0x00000008,
			VMRead = 0x00000010,
			VMWrite = 0x00000020,
			DupHandle = 0x00000040,
			SetInformation = 0x00000200,
			QueryInformation = 0x00000400,
			Synchronize = 0x00100000
		}

		public static IntPtr OpenProcess(ProcessAccessFlags dwDesiredAccess, [MarshalAs(UnmanagedType.Bool)] bool bInheritHandle, int dwProcessId)
		{ return OpenProcess(dwDesiredAccess, bInheritHandle, dwProcessId); }

		/// <summary>
		/// 開いているオブジェクトハンドルを閉じます。
		/// </summary>
		/// <param name="jObject">オブジェクトのハンドル</param>
		/// <returns>関数が成功すると、0 以外の値が返ります。</returns>
		public static int CloseHandle(IntPtr jObject)
		{ return CloseHandle(jObject); }

		
		public static IntPtr LoadLibrary(string lpFileName)
		{ return LoadLibrary(lpFileName); }
		
		public static IntPtr GetProcAddress(IntPtr hModule, string lpProcName)
		{ return GetProcAddress(hModule, lpProcName); }
		
		public static bool FreeLibrary(IntPtr hLibModule)
		{ return FreeLibrary(hLibModule); }

		[Flags]
		public enum AllocationType : uint
		{
			Commit = 0x1000,
			Reserve = 0x2000,
			Decommit = 0x4000,
			Release = 0x8000,
			Reset = 0x80000,
			Physical = 0x400000,
			TopDown = 0x100000,
			WriteWatch = 0x200000,
			LargePages = 0x20000000
		}

		[Flags]
		public enum MemoryProtection : uint
		{
			Execute = 0x10,
			ExecuteRead = 0x20,
			ExecuteReadWrite = 0x40,
			ExecuteWriteCopy = 0x80,
			NoAccess = 0x01,
			ReadOnly = 0x02,
			ReadWrite = 0x04,
			WriteCopy = 0x08,
			GuardModifierflag = 0x100,
			NoCacheModifierflag = 0x200,
			WriteCombineModifierflag = 0x400
		}

		[Flags]
		public enum FreeType : uint
		{
			Decommit = 0x4000,
			Release = 0x8000,
		}

		public static IntPtr VirtualAllocEx(IntPtr hProcess, IntPtr lpAddress, UInt32 dwSize, AllocationType flAllocationType, MemoryProtection flProtect)
		{ return NativeMethods.VirtualAllocEx(hProcess, lpAddress, (UIntPtr)dwSize, (UInt32)flAllocationType, (UInt32)flProtect); }

		public static bool VirtualFreeEx(IntPtr hProcess, IntPtr lpAddress, UInt32 dwSize, FreeType dwFreeType)
		{ return NativeMethods.VirtualFreeEx(hProcess, lpAddress, (UIntPtr)dwSize, (UInt32)dwFreeType); }

		/// <summary>
		/// NativeMethods ネイティブコードを呼び出す内部クラス
		/// </summary>
		internal static class NativeMethods
		{
			[DllImport("kernel32.dll", SetLastError = true, ExactSpelling = true, CharSet = CharSet.Unicode)]
			public static extern uint FormatMessageW(uint dwFlags, IntPtr lpSource, uint dwMessageId, uint dwLanguageId, [MarshalAs(UnmanagedType.LPWStr)] StringBuilder lpBuffer, int nSize, IntPtr Arguments);

			[DllImport("kernel32.dll", SetLastError = true, ExactSpelling = true, CharSet = CharSet.Unicode, EntryPoint = "GetModuleHandleW")]
			public static extern IntPtr GetModuleHandleW([MarshalAs(UnmanagedType.LPWStr)] string lpModuleName);

			[DllImport("kernel32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern bool ReadProcessMemory(IntPtr hProcess, IntPtr lpBaseAddress, [System.Runtime.InteropServices.Out] byte[] lpBuffer, UIntPtr dwSize, out UInt32 lpNumberOfBytesRead);

			[DllImport("kernel32", SetLastError = true, ExactSpelling = true)]
			public static extern int CloseHandle(IntPtr jObject);

			[DllImport("kernel32.dll", SetLastError = true, ExactSpelling = true, CharSet = CharSet.Unicode, EntryPoint = "LoadLibraryW")]
			public static extern IntPtr LoadLibraryW([MarshalAs(UnmanagedType.LPWStr)] string lpFileName);

			[DllImport("kernel32.dll", SetLastError = true, ExactSpelling = true, BestFitMapping = false, ThrowOnUnmappableChar = true)]
			public static extern IntPtr GetProcAddress(IntPtr hModule, [MarshalAs(UnmanagedType.LPStr)] string lpProcName);

			[DllImport("kernel32.dll", SetLastError = true)]
			public static extern bool FreeLibrary(IntPtr hLibModule);

			[DllImport("kernel32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern IntPtr VirtualAllocEx(IntPtr hProcess, IntPtr lpAddress, UIntPtr dwSize, UInt32 flAllocationType, UInt32 flProtect);

			[DllImport("kernel32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern bool VirtualFreeEx(IntPtr hProcess, IntPtr lpAddress, UIntPtr dwSize, UInt32 dwFreeType);
		}
	}
}
