﻿using System;
using System.Drawing;
using System.Text;

using System.Runtime.InteropServices;

namespace Win32SDK
{
	internal static class User32
	{
		public static readonly IntPtr HWND_NULL = IntPtr.Zero;
		public static readonly IntPtr HWND_TOPMOST = new IntPtr(-1);
		public static readonly IntPtr HWND_NOTOPMOST = new IntPtr(-2);

		public const int SWP_NOSIZE = 0x0001;
		public const int SWP_NOMOVE = 0x0002;
		public const int SWP_NOACTIVATE = 0x0010;
		public const int SWP_SHOWWINDOW = 0x0040;

		public const int WS_EX_LAYERED = 0x00080000;
		public const int WS_EX_TRANSPARENT = 0x00000020;
		public const int WS_BORDER = 0x00800000;
		public const int WS_THICKFRAME = 0x00040000;

		public const int ULW_ALPHA = 2;

		public const byte AC_SRC_OVER = 0;
		public const byte AC_SRC_ALPHA = 1;

		public const int WH_KEYBOARD_LL = 13;

		public static int SC_MAXIMIZE = 0xf030;

		public static int HTCLIENT = 1;
		public static int HTCAPTION = 2;

		public const int HC_ACTION = 0;

		/// <summary>
		/// RECT 構造体
		/// </summary>
		[StructLayout(LayoutKind.Sequential)]
		public struct RECT
		{
			/// <summary>
			/// 四角形の左上隅の x 座標を指定します。
			/// </summary>
			public int left;
			/// <summary>
			/// 四角形の左上隅の y 座標を指定します。
			/// </summary>
			public int top;
			/// <summary>
			/// 四角形の右下隅の x 座標を指定します。
			/// </summary>
			public int right;
			/// <summary>
			/// 四角形の右下隅の y 座標を指定します。
			/// </summary>
			public int bottom;
		}

		/// <summary>
		/// BLENDFUNCTION 構造体
		/// </summary>
		[StructLayout(LayoutKind.Sequential, Pack = 1)]
		public struct BLENDFUNCTION
		{
			/// <summary>
			/// ブレンド操作(常にAC_SRC_OVER(=0))
			/// </summary>
			public byte BlendOp;
			/// <summary>
			/// 常に0
			/// </summary>
			public byte BlendFlags;
			/// <summary>
			/// コピー元のビットマップ全体に適用するアルファ値
			/// </summary>
			public byte SourceConstantAlpha;
			/// <summary>
			/// コピー元のビットマップがアルファ値を持つとき(AC_SRC_ALPHA=1)
			/// </summary>
			public byte AlphaFormat;
		}

		/// <summary>
		/// キーボードフックプロシージャ LPARAM 用構造体
		/// </summary>
		[System.Runtime.InteropServices.StructLayout(LayoutKind.Sequential)]
		public struct KBDLLHOOKSTRUCT
		{
			public UInt32 vkCode;
			public UInt32 scanCode;
			public UInt32 flags;
			public UInt32 time;
			public IntPtr dwExtraInfo;
		}

		/// <summary>
		/// Win32SDK GetDC : 指定されたウィンドウのクライアント領域または
		/// 画面全体を表すディスプレイデバイスコンテキストのハンドルを取得します。
		/// その後、GDI 関数を使って、返されたデバイスコンテキスト内で描画を行えます。
		/// </summary>
		/// <param name="hWnd">ウィンドウのハンドル</param>
		/// <returns>関数が成功すると、指定したウィンドウのクライアント領域を表すデバイスコンテキストのハンドルが返ります。</returns>
		public static IntPtr GetDC(IntPtr hWnd)
		{ return NativeMethods.GetDC(hWnd); }

		/// <summary>
		/// Win32SDK ReleaseDC : デバイスコンテキストを解放し、他のアプリケーションからつかえるようにします。
		/// </summary>
		/// <param name="hWnd">解放対象のデバイスコンテキストに対応するウィンドウのハンドルを指定します</param>
		/// <param name="hDC">解放対象のデバイスコンテキストのハンドルを指定します</param>
		/// <returns>関数が成功すると、0 以外の値が返ります。</returns>
		public static Int32 ReleaseDC(IntPtr hWnd, IntPtr hDC)
		{ return NativeMethods.ReleaseDC(hWnd, hDC); }

		/// <summary>
		/// Win32SDK GetWindowDC : タイトルバー、メニュー、スクロールバーを含む、ウィンドウ全体のデバイスコンテキスト（DC）を取得します。
		/// </summary>
		/// <param name="hwnd">デバイスコンテキストを取得したいウィンドウのハンドルを指定します。NULL を指定すると、GetWindowDC は画面全体のデバイスコンテキストを取得します。</param>
		/// <returns>関数が成功すると、0 以外の値が返ります。</returns>
		public static IntPtr GetWindowDC(IntPtr hwnd)
		{ return NativeMethods.GetWindowDC(hwnd); }

		/// <summary>
		/// Win32SDK GetForegroundWindow : フォアグラウンドウィンドウ（ 現在ユーザーが作業しているウィンドウ）のハンドルを返します。
		/// </summary>
		/// <returns></returns>
		public static IntPtr GetForegroundWindow()
		{ return NativeMethods.GetForegroundWindow(); }

		/// <summary>
		/// Win32SDK GetWindowRect : 指定されたウィンドウの左上端と右下端の座標をスクリーン座標で取得します。
		/// スクリーン座標は、表示画面の左上端が (0,0) となります。
		/// </summary>
		/// <param name="hwnd"ウィンドウのハンドルを指定します></param>
		/// <param name="lpRect">構造体へのポインタを指定します。この構造体の left メンバと top メンバに、スクリーン座標でのウィンドウ左上端の座標が入ります。right メンバと bottom メンバには、ウィンドウの右下端の座標が入ります。</param>
		/// <returns>関数が成功すると、0 以外の値が返ります。</returns>
		public static Int32 GetWindowRect(IntPtr hwnd, [System.Runtime.InteropServices.In()] ref RECT lpRect)
		{ return NativeMethods.GetWindowRect(hwnd, ref lpRect); }

		/// <summary>
		/// Win32SDK UpdateLayeredWindow : レイヤードウィンドウの位置、サイズ、形、内容、透明度を更新します。
		/// </summary>
		/// <param name="hwnd">レイヤードウィンドウのハンドル</param>
		/// <param name="hdcDst">画面のデバイスコンテキストのハンドル</param>
		/// <param name="pptDst">画面の新しい位置</param>
		/// <param name="psize">レイヤードウィンドウの新しいサイズ</param>
		/// <param name="hdcSrc">サーフェスのデバイスコンテキストのハンドル</param>
		/// <param name="pptSrc">レイヤの位置</param>
		/// <param name="crKey">カラーキー</param>
		/// <param name="pblend">ブレンド機能</param>
		/// <param name="dwFlags">フラグ</param>
		/// <returns>関数が成功すると、0 以外の値が返ります。</returns>
		public static Int32 UpdateLayeredWindow(
			IntPtr hwnd,
			IntPtr hdcDst,
			[System.Runtime.InteropServices.In()]
            ref Point pptDst,
			[System.Runtime.InteropServices.In()]
            ref Size psize,
			IntPtr hdcSrc,
			[System.Runtime.InteropServices.In()]
            ref Point pptSrc,
			int crKey,
			[System.Runtime.InteropServices.In()]
            ref BLENDFUNCTION pblend,
			int dwFlags)
		{ return NativeMethods.UpdateLayeredWindow(hwnd, hdcDst, ref pptDst, ref psize, hdcSrc, ref pptSrc, crKey, ref pblend, dwFlags); }

		/// <summary>
		/// Win32SDK SetWindowPos : 子ウィンドウ、ポップアップウィンドウ、または
		/// トップレベルウィンドウのサイズ、位置、および Z オーダーを変更します。
		/// </summary>
		/// <param name="hWnd">ウィンドウのハンドル</param>
		/// <param name="hWndInsertAfter">配置順序のハンドル</param>
		/// <param name="x">横方向の位置</param>
		/// <param name="y">縦方向の位置</param>
		/// <param name="width">幅</param>
		/// <param name="height">高さ</param>
		/// <param name="flags">ウィンドウ位置のオプション</param>
		/// <returns>関数が成功すると、0 以外の値が返ります。</returns>
		public static UInt32 SetWindowPos(IntPtr hWnd, IntPtr hWndInsertAfter, int x, int y, int width, int height, int flags)
		{ return NativeMethods.SetWindowPos(hWnd, hWndInsertAfter, x, y, width, height, flags); }

		/// <summary>
		/// SetWindowsHookEx 関数と共に使われる、アプリケーション定義またはライブラリ定義のコールバック関数です。
		/// 新しいキーボード入力イベントをスレッドの入力キューへポストしようとするときに、システムは必ずこのフックプロシージャを呼び出します。
		/// キーボード入力を発生させるのは、ローカルのキーボードドライバまたは、 関数です。
		/// keybd_event を呼び出した場合、キー入力がエミュレートされ、特定のキーを押したのと同様のイベントが発生します。
		/// </summary>
		/// <param name="nCode">nCode パラメータの値が 0 未満の場合、このフックプロシージャはメッセージを処理せずにそのメッセージを CallNextHookEx 関数へ渡し、その関数の戻り値を返さなければなりません。</param>
		/// <param name="wParam">キーボードメッセージの識別子を指定します。このパラメータでいずれかのメッセージを指定します。</param>
		/// <param name="lParam">KBDLLHOOKSTRUCT 構造体</param>
		/// <returns>code パラメータの値が 0 未満の場合、このフックプロシージャは CallNextHookEx. 関数を呼び出し、その関数の戻り値を返さなければなりません。</returns>
		[System.Runtime.InteropServices.UnmanagedFunctionPointer(System.Runtime.InteropServices.CallingConvention.Cdecl)]
		public delegate IntPtr LowLevelKeyboardProc(int nCode, IntPtr wParam, ref KBDLLHOOKSTRUCT lParam);

		/// <summary>
		/// アプリケーション定義のフックプロシージャをフックチェーン内にインストールします。フックプロシージャをインストールすると、特定のイベントタイプを監視できます。監視の対象になるイベントは、特定のスレッド、または呼び出し側スレッドと同じデスクトップ内のすべてのスレッドに関連付けられているものです。
		/// </summary>
		/// <param name="idHook">フックタイプ</param>
		/// <param name="lpfn">フックプロシージャ</param>
		/// <param name="hMod">アプリケーションインスタンスのハンドル</param>
		/// <param name="dwThreadId">スレッドの識別子</param>
		/// <returns>関数が成功すると、フックプロシージャのハンドルが返ります。関数が失敗すると、NULL が返ります。拡張エラー情報を取得するには、 関数を使います。</returns>
		public static IntPtr SetWindowsHookEx(int idHook, LowLevelKeyboardProc lpfn, IntPtr hMod, int dwThreadId)
		{ return NativeMethods.SetWindowsHookEx(idHook, lpfn, hMod, dwThreadId); }

		/// <summary>
		/// 現在のフックチェーン内の次のフックプロシージャに、フック情報を渡します。フックプロシージャは、フック情報を処理する前でも、
		/// フック情報を処理した後でも、この関数を呼び出せます。
		/// </summary>
		/// <param name="hHook">現在のフックのハンドルを指定します。アプリケーションは、
		/// SetWindowsHookEx 関数を呼び出した際に取得したハンドルを指定します。</param>
		/// <param name="nCode">現在のフックプロシージャに渡されたフックコードを指定します。次のフックプロシージャはこのフックコードを使って、フック情報の処理方法を決定します。</param>
		/// <param name="wParam">現在のフックプロシージャに渡された wParam 値を指定します。このパラメータの意味は、現在のフックチェーンに関連付けられているフックタイプに依存します。</param>
		/// <param name="lParam">現在のフックプロシージャに渡された lParam 値を指定します。このパラメータの意味は、現在のフックチェーンに関連付けられているフックタイプに依存します。</param>
		/// <returns>フックチェーン内の次のフックプロシージャの戻り値が返ります。現在のフックプロシージャは、この値をそのまま戻り値として返さなければなりません。この値の意味は、フックの種類に依存します。詳細については、各フックプロシージャの説明を参照してください。</returns>
		public static IntPtr CallNextHookEx(IntPtr hHook, int nCode, IntPtr wParam, ref KBDLLHOOKSTRUCT lParam)
		{ return NativeMethods.CallNextHookEx(hHook, nCode, wParam, ref lParam); }

		/// <summary>
		/// SetWindowsHookEx 関数を使ってフックチェーン内にインストールされたフックプロシージャを削除します。
		/// </summary>
		/// <param name="hHook">削除対象のフックプロシージャのハンドルを指定します。このハンドルは、以前に SetWindowsHookEx 関数を呼び出したときに取得したフックのハンドルです。</param>
		/// <returns>関数が成功すると、0 以外の値が返ります。</returns>
		public static bool UnhookWindowsHookEx(IntPtr hHook)
		{ return NativeMethods.UnhookWindowsHookEx(hHook); }

		/// <summary>
		/// 関数呼び出し時にキーが押されているかどうか、また、前回の GetAsyncKeyState 関数呼び出し以降にキーが押されたかどうかを判定します。
		/// </summary>
		/// <param name="vKey">仮想キーコードのいずれかを指定します。</param>
		/// <returns>関数が成功すると、前回の GetAsyncKeyState 関数呼び出し以降にキーが押されたかどうか、およびキーが現在押されているかどうかを示す値が返ります。最上位ビットがセットされたときは現在そのキーが押されていることを示し、最下位ビットがセットされたときは前回の GetAsyncKeyState 関数呼び出し以降にそのキーが押されたことを示します。他のスレッド（ またはプロセス）のウィンドウがキーボードフォーカスを備えている場合は、0 が返ります。</returns>
		public static Int16 GetAsyncKeyState(Int32 vKey)
		{ return NativeMethods.GetAsyncKeyState(vKey); }

		/// <summary>
		/// 指定されたウィンドウを作成したスレッドの ID を取得します。必要であれば、ウィンドウを作成したプロセスの ID も取得できます。
		/// </summary>
		/// <param name="hWnd">ウィンドウのハンドル</param>
		/// <param name="lpdwProcessId">プロセス ID</param>
		/// <returns>ウィンドウを作成したスレッドの ID が返ります。</returns>
		public static uint GetWindowThreadProcessId(IntPtr hWnd, out int lpdwProcessId)
		{ return NativeMethods.GetWindowThreadProcessId(hWnd, out lpdwProcessId); }

		/// <summary>
		/// ネイティブコードを呼び出すクラス
		/// </summary>
		internal static class NativeMethods
		{
			[DllImport("user32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern IntPtr GetDC(IntPtr hWnd);

			[DllImport("user32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern Int32 ReleaseDC(IntPtr hWnd, IntPtr hDC);

			[DllImport("user32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern IntPtr GetWindowDC(IntPtr hwnd);

			[DllImport("user32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern IntPtr GetForegroundWindow();

			[DllImport("user32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern Int32 GetWindowRect(IntPtr hwnd, [System.Runtime.InteropServices.In()] ref RECT lpRect);

			[DllImport("user32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern Int32 UpdateLayeredWindow(
				IntPtr hwnd,
				IntPtr hdcDst,
				[System.Runtime.InteropServices.In()] ref Point pptDst,
				[System.Runtime.InteropServices.In()] ref Size psize,
				IntPtr hdcSrc,
				[System.Runtime.InteropServices.In()] ref Point pptSrc,
				int crKey,
				[System.Runtime.InteropServices.In()] ref BLENDFUNCTION pblend,
				int dwFlags);

			[DllImport("user32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern UInt32 SetWindowPos(IntPtr hWnd, IntPtr hWndInsertAfter, int x, int y, int width, int height, int flags);

			//[DllImport("user32.dll", SetLastError = true, ExactSpelling = true, CharSet = CharSet.Unicode)]
			[DllImport("user32.dll", EntryPoint="SetWindowsHookExA", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
			public static extern IntPtr SetWindowsHookEx(int idHook, LowLevelKeyboardProc lpfn, IntPtr hMod, int dwThreadId);

			[DllImport("user32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern IntPtr CallNextHookEx(IntPtr hHook, int nCode, IntPtr wParam, ref KBDLLHOOKSTRUCT lParam);

			[DllImport("user32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern bool UnhookWindowsHookEx(IntPtr hHook);

			[DllImport("user32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern Int16 GetAsyncKeyState(Int32 vKey);

			[DllImport("user32.dll", SetLastError = true, ExactSpelling = true)]
			public static extern uint GetWindowThreadProcessId(IntPtr hWnd, out int lpdwProcessId);
		}
	}
}
