﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Threading;

namespace PluginBase
{
	/// <summary>
	/// イベントキュークラス
	/// .NET リモートでのイベント送信は大げさと判断して、クライアント側はこのキューを監視する形でサーバー側のイベントを処理します。
	/// （スレッドセーフです）
	/// </summary>
	public class EventQueue<T>
	{
		/// <summary>
		/// ロック用オブジェクト
		/// </summary>
		private object _lockObj = new object();
		/// <summary>
		/// キュー本体
		/// </summary>
		private Queue<T> _queue = new Queue<T>();
		

		/// <summary>
		/// 標準のコンストラクタ
		/// </summary>
		public EventQueue()
		{
		}

		/// <summary>
		/// イベントをキューへプッシュします。
		/// </summary>
		/// <param name="appEvent"></param>
		public void Push(T appEvent)
		{
			lock(_lockObj)
			{
				_queue.Enqueue(appEvent);
			}
		}

		/// <summary>
		/// イベントをキューからポップします。
		/// ポップされたイベントはキューから削除されます。
		/// </summary>
		/// <param name="appEvent"></param>
		/// <returns></returns>
		public bool Pop(out T appEvent)
		{
			appEvent = default(T);

			lock(_lockObj)
			{
				if(_queue.Count > 0)
				{
					appEvent = _queue.Dequeue();
					return true;
				}

				return false;
			}
		}
	}
}
