﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PluginBase
{
	/// <summary>
	/// シングルトン化のためのヘルパークラス
	/// クラス T をシングルトン化します。
	/// </summary>
	/// <typeparam name="T">シングルトン化するクラスの型</typeparam>
	public class SingletonHelperBase<T> : MarshalByRefObject, ISingletonHelper where T : IPlugin, new()
	{
		/// <summary>
		/// 唯一のインスタンス
		/// </summary>
		private readonly static T _plugin = new T();

		/// <summary>
		/// リース期間を無期限に設定
		/// </summary>
		/// <returns></returns>
		public override object InitializeLifetimeService()
		{
			return null;
		}

		/// <summary>
		/// スタティックコンストラクタ
		/// </summary>
		static SingletonHelperBase()
		{
		}

		/// <summary>
		/// プラグインのインスタンスを取得
		/// </summary>
		/// <returns></returns>
		public IPlugin Instance
		{
			get { return _plugin; }
		}
	}
}
