﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.IO;
using System.Drawing;

namespace PluginBase
{
	/// <summary>
	/// 再生中の曲情報を格納するクラス
	/// </summary>
	[Serializable]
	public class TrackInfo : IDisposable
	{
		/// <summary>
		/// ファイルパス
		/// </summary>
		public string FilePath { set; get; }

		/// <summary>
		/// トラック名の取得、設定
		/// </summary>
		public string Name { set; get; }
		/// <summary>
		/// アーティスト名の取得、設定
		/// </summary>
		public string Artist { set; get; }
		/// <summary>
		/// アルバム名の取得、設定
		/// </summary>
		public string Album { set; get; }
		/// <summary>
		/// アートワークがあるファイルパスの取得、設定
		/// (リモートで行うためファイル経由で行う)
		/// </summary>
		public string ArtworkFilepath { set; get; }
		/// <summary>
		/// トラック番号の取得、設定
		/// </summary>
		public string TrackNo { set; get; }
		/// <summary>
		/// 総再生時間[s]の取得、設定
		/// </summary>
		public int Duration { set; get; }

		/// <summary>
		/// コンストラクタ
		/// </summary>
		public TrackInfo()
		{
			FilePath = "";
	
			Name = "";
			Artist = "";
			Album = "";
			TrackNo = "";
			ArtworkFilepath = "";
			Duration = 0;
		}

		/// <summary>
		/// デストラクタ
		/// </summary>
		~TrackInfo()
		{
			Dispose(false);
		}

		/// <summary>
		/// リソースの開放処理
		/// </summary>
		public void Dispose()
		{
			//マネージリソースおよびアンマネージリソースの解放
			this.Dispose(true);
			//ガベコレから、このオブジェクトのデストラクタを対象外とする
			GC.SuppressFinalize(this);
		}

		protected virtual void Dispose(bool disposing)
		{
		}

		/// <summary>
		/// クリア
		/// </summary>
		public void Clear()
		{
			FilePath = "";

			Name = "";
			Artist = "";
			Album = "";
			TrackNo = "";
			ArtworkFilepath = "";
			Duration = 0;

			Dispose();
		}

		/// <summary>
		/// 書式にしたがって文字列を生成します。
		/// </summary>
		/// <param name="format">書式</param>
		/// <returns></returns>
		public string Format(string format)
		{
			string result = format;

			result = result.Replace("%A", Artist);
			result = result.Replace("%a", Artist);

			result = result.Replace("%L", Album);
			result = result.Replace("%l", Album);

			result = result.Replace("%I", TrackNo);
			result = result.Replace("%i", TrackNo);

			result = result.Replace("%T", Name);
			result = result.Replace("%t", Name);

			result = result.Replace("%N", "\n");
			result = result.Replace("%n", "\n");

			return result;
		}

		/// <summary>
		/// ディープコピー
		/// </summary>
		/// <returns></returns>
		public TrackInfo DeepCopy()
		{
			TrackInfo result = (TrackInfo)MemberwiseClone();

			return result;
		}
	}
}
